!****************************************************************************
      module comm_interface

      implicit none
      include "mpif.h"
      public comm_init          ! Initialize MPI
      public comm_barrier       ! MPI barrier
      public comm_finalize      ! Terminate MPI
      public comm_gather
      integer, public  :: iam
      integer, public  :: nproc
      SAVE
      private
      integer :: mpicom

      CONTAINS

!-----------------------------------------------------------------------------
      subroutine comm_init()
      implicit none
      integer :: ier

      mpicom = MPI_COMM_WORLD
      call mpi_init(ier)
      call mpi_comm_rank(mpicom, iam, ier)
      call mpi_comm_size(mpicom, nproc, ier)

      return
      end subroutine comm_init
!-----------------------------------------------------------------------------
      subroutine comm_barrier()
      implicit none
      integer :: ier

      call mpi_barrier(mpicom, ier)

      return
      end subroutine comm_barrier
!-----------------------------------------------------------------------------

       subroutine comm_finalize()
      implicit none
      integer :: ier

      call mpi_finalize(ier)

      return
       end subroutine comm_finalize
!-----------------------------------------------------------------------------
      subroutine comm_gather(s, sc, r,rc )
      integer :: ier,sc, rc
      integer,intent(in):: s(2)
      integer,intent(out):: r(2*nproc)

      call mpi_gather(s,sc,MPI_INT,r,rc,MPI_INT,0,mpicom,ier)
 
      return 
      end subroutine comm_gather
!-----------------------------------------------------------------------------
      end module comm_interface
!*****************************************************************************
      module param1
      implicit real*8(a-h,o-z)
      INCLUDE 'PARAM'
      end module param1

      module pdam
      real*8, allocatable :: arrayda(:),matda(:,:)
      end module pdam

      module mod_namelist
      integer :: IMEMDIR=0,IRECUPH=1,IRECUPD=1,IDELETE=0
      end module  mod_namelist          

!***********************************************************************
!***********************************************************************
      PROGRAM PDTO3_dip
       use mod_namelist
       use comm_interface, only : iam,nproc,comm_init,comm_barrier,
     A                           comm_finalize	
      use param1                 
C
C      Nicole Dunleavy , Connor Ballance  sept 20  2019
C       
C      NAMELIST introduced, RECUPH/RECUPD or both +  plus memory or
C                           direct file construction (ddto3)
C
C      Deafault options reproduce previous versions
C
c based-on kab's d2h of Dec 2002.
c
c  Converts DARC to RMBP file format: DSTG2.DAT to RECUPH.DAT and RECUPD.DAT.

c  ifile = unit number for DSTG2.DAT input
c  INAST = number of N+1 electron symmetries on file: needs to be defined
C          but can be set large, e.g. 999, to obtain all (stg3 moans a bit) NRB.
c  nproc = number of RECUPH.DAT files to be written (=1 for serial codes).

      logical ok,ok2
      data ifile/30/, INAST/0/, iout/300/
      CHARACTER NAME*3,NUM(0:9)*1
      CHARACTER*9 filbb
      CHARACTER*10 SCRAT,filbc
      DATA NUM /'0','1','2','3','4','5','6','7','8','9'/
      NAMELIST/DDTO3/IMEMDIR,IDELETE,IRECUPH,IRECUPD 
c
c
C
        call comm_init()
C
C        IFLAGERR=0
C
        if(iam.eq.0)then
        call cpu_time(timei)
        endif
c
c

      INAST=2
      IMEMDIR=0
      IDELETE=0
      IRECUPH=1
      IRECUPD=1
C
C      pairs of dipoles           
C
c Open dstg2.f file for reading:
c
      i1=iam/100
      i2=(iam-100*(iam/100))/10
      i3=iam-(100*(iam/100))-i2*10
      filbb='DSTG2.'//NUM(i1)//NUM(i2)//NUM(i3)
      filbc='dto3dip'//NUM(i1)//NUM(i2)//NUM(i3)
      INQUIRE (FILE=filbb,EXIST=ok)

      if (.not.ok) then
        print*,'STOP: file',filbb,'does not exist'
        stop
      endif
      OPEN(UNIT=ifile,STATUS='OLD',FORM='UNFORMATTED',FILE=filbb)
      OPEN(UNIT=iout,STATUS='UNKNOWN',FORM='FORMATTED',FILE=filbc)
      nproca=nproc

      INQUIRE (FILE='ddto3',EXIST=ok2)

      if (ok2) then
       OPEN(444,file='ddto3',form='formatted',status='unknown')
       READ(444,ddto3)
      CLOSE(444)
      else
       continue
      endif

      IF(IMEMDIR.EQ.0) CALL DTO3MEM(INAST,nproca,ifile)
      IF(IMEMDIR.EQ.1) CALL DTO3DIR(INAST,nproca,ifile)
C
      call comm_barrier()
C
      if(iam.eq.0)then
       call cpu_time(timef)
      time=timef-timei
      write(iout,999) time/60.,nproc
999   format(//1x,'CPU TIME=',f9.3,' MIN  -- processors=:',i4)
C
      endif
C 
      call comm_finalize()
C
      STOP
      end
c-----------------------------------------------------------------------
      SUBROUTINE DTO3MEM(INAST,nproca,IFILE)
      use mod_namelist, only : IMEMDIR,IRECUPH,IRECUPD,IDELETE         

c  Converts DARC to RMBP file format: DSTG2.DAT to RECUPH.DAT and RECUPD.DAT.

c  The following need to be predefined:
c  INAST = number of N+1 electron symmetries on file
c  ifile = unit number for DSTG2.DAT input
c  nproc = number of RECUPH.DAT files to be written (=1 for serial codes).

      use comm_interface, only : iam,nproc,comm_barrier,comm_gather
      use param1 
      IMPLICIT REAL*8(A-H,O-Z)

c  DSTG2.DAT is the file from the DARC program dstg2.f, 
c  and can contain both Hamiltonian and dipole matrix elements.

c  The DTO3 routine reads this file and creates file RECUPH.DAT with
c  the Hamiltonians, and RECUPD.DAT with the dipoles if present,
c  in the format required by the BPRM program stg3r.f
c  (the format was defined by Berrington et al 1995 in CPC 92,290-420).

c  It also converts the Buttle correction to 'MJS' style.
c  It also detects any external-region contributions to the dipole matrix
c  (these can now be calculated in dstg2.f with some mods by kab).

c   DARC style Buttle correction written to DBUT.DAT (case MJS=0) for
c   automatic read by stgf.

c Include the dimension parameters MZLR2,MZNR2,MZNC2,MZLMX,MZCHF,MZSLP,MZTAR
c using the Badnell (stg3r.f etc) convention:
C
C      INCLUDE 'PARAM'
C
c This could be done using the DARC dstg2.f dimension plants as follows:
c      PARAMETER (MZLR2={NK})
c      PARAMETER (MZNR2={NB})
c      PARAMETER (NXNC2={NC})
c      PARAMETER (MZCHF={CH})
c      PARAMETER (MZTAR={NL})
c      PARAMETER (MZLMX=4)
c      PARAMETER (MZSLP=100)

c Derive further parameters to set dimensions.
c The biggest array here is work, whose size=MZNR2*MZNC2*MZCHF*MZCHF/2.

c      PARAMETER (INML=MZNR2/MZNC2, JNML=MZNC2/MZNR2, KNML=INML+JNML)
c      PARAMETER (ND24=MZNR2*INML/KNML+MZNC2*JNML/KNML)
c      PARAMETER (kccc=(MZCHF*(MZCHF+1))/2)
      PARAMETER (NBUT=2*(MZNR2-1))
      parameter (zero=0.0)

      CHARACTER IHED*80,RECORD*20
      CHARACTER*1 NUM(0:9)
      CHARACTER*9 REC,RECD
      CHARACTER*10 SCRAT

      logical outer,bnew

      DIMENSION MAXNQN(MZLR2), MINNQN(MZLR2),
     *          MAXFUL(MZLR2), LAG(MZLR2), KAG(MZLR2), JAG(MZLR2),
     *          NCONAT(MZTAR), K2P(MZCHF), L2P(MZCHF),
     *          MBUT(MZLR2), ksym(MZCHF,MZCHF),
     *          ILSPI(3,MZSLP), temp(MZLR2)
      DIMENSION AC(MZCHF,MZCHF), BLC(MZCHF,MZCHF), BVC(MZCHF,MZCHF),
     *          EBUT(NBUT,MZLR2),CBUT(NBUT,MZLR2),CF(MZCHF,MZCHF,MZLMX),
     *          e(NBUT), f(NBUT), eigmax(MZLR2)
C     *          wrkcb(MZNR2,MZNC2,MZCHF),wrkbb(MZNC2)
C
      integer,allocatable :: bp1(:),bp2(:),bp3(:),bp4(:),bp5(:)
C      
       real*8,allocatable :: wrkcc(:,:,:)
       real*8,allocatable :: wrkd(:,:,:)
       real*8,allocatable :: wrkcb(:,:,:),wrkbb(:)
C
      COMMON/COM1/ENDS(MZNR2,MZLR2),ENAT(MZTAR),
     *            EIGENS(MZNR2,MZLR2),COEFF(3,MZLR2)
      COMMON/COM2/JAT(MZTAR),LPTY(MZTAR), MAXNLG(MZLR2)
      COMMON/COM3/NELC,NZ,LRANG1,LRANG2,NRANG2,LAMAX,ICODE,LAM,IZESP,
     *            JRELOP,maxnc,NAST,isat,NIX,RA,BSTO,HINT,eta

c  MJS   = 1 (default) for producing Buttle fit;
c        = 0 for dump on DBUT.DAT (this also occurs if fit fails).
c  outer = true (default) for outer region contributions, if any;
c        = false for inserting zeros.
c  jfile = 32 (default) unit number for RECUPD.DAT output.
c  kfile = 33 (default) unit number for RECUPH.DAT output.
c  iprint= 1 (default) for printing useful stuff on standard output;
c        = 0 for emergency printing only.
c  The other parameters set here should be acceptable for BPRM.

c      data MJS/1/, outer/.TRUE./, jfile/32/, kfile/33/, iprint/1/
c      data ICODE/25/, IZESP/0/, JRELOP/1/, maxnc/0/, isat/0/, NSPN/0/
c      data ABUTL/0.0/, ABUTV/0.0/, BBUTL/0.0/, BBUTV/0.0/, eta/0.0/
      DATA NUM/'0','1','2','3','4','5','6','7','8','9'/
      DATA iout/300/
      INTEGER :: bpv1(2),bpv2(2),bpv3(2),bpv4(2),bpv5(2)
c
      MJS=1
      outer=.TRUE.
      ifile=30
      jfile2=29
      jfile=32
      kfile=33
      iprint=1
      ICODE=25
      JRELOP=1
      IZESP=0
      maxnc=0
      isat=0
      NSPN=0
      ABUTL=0.0d0
      ABUTV=0.0d0
      BBUTL=0.0d0
      BBUTV=0.0d0
      eta=0.0d0
c
c  Check dimensions for number of symmetries input

      if( INAST.gt.MZSLP ) then
        print*,'STOP! increase dimension MZLSP to:',MZSLP
        stop
      endif

c
c nrec=number of symmetries per file
c
C      nrec=inast/nproc
      nrec=2                 !pair of dipoles per file
C      if(nrec.le.0)nrec=1
C
      iam1=iam*2
      iam2=iam1+1
C
      i1=iam1/100
      i2=(iam1-100*(iam1/100))/10
      i3=iam1-(100*(iam1/100))-i2*10
      REC='RECUPH'//NUM(i1)//NUM(i2)//NUM(i3)
c  Open Hamiltonian output file and info file
        OPEN( UNIT=jfile, STATUS='UNKNOWN', FORM='UNFORMATTED',
     *      FILE=REC)

      i1=iam2/100
      i2=(iam2-100*(iam2/100))/10
      i3=iam2-(100*(iam2/100))-i2*10
      REC='RECUPH'//NUM(i1)//NUM(i2)//NUM(i3)
c  Open Hamiltonian output file and info file
        OPEN( UNIT=jfile2, STATUS='UNKNOWN', FORM='UNFORMATTED',
     *      FILE=REC)
c-----------------------------------------------------------------------
c Read header from DSTG2.DAT file

c Some of the variables are DARC-specific and do not seem to correspond
c to anything in BPRM, eg speed of light CL, so we will try to ignore them.

c On the other hand, there are some variables that BPRM would like,
c which the DARC file does not have, eg. LAMAX, so we will try to derive them.

c Along the way, we will also check dimensions.

c      rewind ifile
      READ (ifile) IHED,RECORD
      if(iprint.ge.1)write(iout,*)IHED,RECORD
      READ (ifile) KBMAX,KCMIN,KCMAX,KX,NRANG2,NELC,NZ
      READ (ifile) LAMBB,LAMBC,LAMCC

      LRANG1 = KX
      LRANG2 = KCMAX                ! but starts at KCMIN
      LAMAX  = LAMBB
      LAM    = 1
      if( LAMBC.gt.0 ) LAM = 2
      if( LAMCC.gt.0 ) LAM = 3

      if( LRANG1.gt.MZLR2 .or. LRANG2.gt.MZLR2  ) then
        print*,'STOP! Increase dimension MZLR2 to:',max(LRANG1,LRANG2)
        stop
      endif
      if( LAMAX.gt.MZLMX ) then
        print*,'STOP! Increase dimension MZLMX to:',LAMAX
        stop
      endif
      if( NRANG2.gt.MZNR2 ) then
        print*,'STOP! Increase dimension MZNR2 to:',NRANG2
        stop
      endif

      IF( KBMAX.GT.0 ) THEN
        READ (ifile) (MINNQN(L),L=1,KX), (MAXNQN(L),L=1,KX),
     *               (MAXNLG(L),L=1,KX), (MAXFUL(L),L=1,KBMAX),
     *               (LAG(L),L=1,KX), (KAG(L),L=1,KX), (JAG(L),L=1,KX)
      ELSE
        READ (ifile) (MINNQN(L),L=1,KX), (MAXNQN(L),L=1,KX),
     *               (MAXNLG(L),L=1,KX), (MAXFUL(L),L=1,KX),
     *               (LAG(L),L=1,KX),(KAG(L),L=1,KX), (JAG(L),L=1,KX)
      ENDIF

      READ (ifile) RA,BSTO,HINT,CL,DR1
      READ (ifile) NIX
      READ (ifile) (IRX,I=1,NIX)
      READ (ifile) NUMORB

c Write header to standard RECUPH.DAT hamilitonian output file

        WRITE(jfile) NELC,NZ,LRANG1,LRANG2,NRANG2,LAMAX,ICODE,LAM,IZESP,
     *               JRELOP,JRELOP,JRELOP
        WRITE(jfile) (MAXNLG(L),L=1,LRANG1), (MAXNLG(L),L=1,LRANG1),
     *               (maxnc,L=1,LRANG1)
        WRITE(jfile2)NELC,NZ,LRANG1,LRANG2,NRANG2,LAMAX,ICODE,LAM,IZESP,
     *               JRELOP,JRELOP,JRELOP
        WRITE(jfile2) (MAXNLG(L),L=1,LRANG1), (MAXNLG(L),L=1,LRANG1),
     *               (maxnc,L=1,LRANG1)


      IF (KCMIN.GT.1) THEN
        DO L = 1,KCMIN-1
          DO N=1,NRANG2
            ENDS(N,L)=ZERO
            EIGENS(N,L)=ZERO
          ENDDO
            WRITE(jfile) (EIGENS(N,L),N=1,NRANG2)
            WRITE(jfile) (ENDS(N,L),N=1,NRANG2)
            WRITE(jfile2) (EIGENS(N,L),N=1,NRANG2)
            WRITE(jfile2) (ENDS(N,L),N=1,NRANG2)
        ENDDO
      ENDIF
c Finish reading header

      DO L = KCMIN,KCMAX
        READ (ifile) (EIGENS(N,L),N=1,NRANG2)
        WRITE(jfile) (2*EIGENS(N,L),N=1,NRANG2) !a.u. to Ryd
        WRITE(jfile2) (2*EIGENS(N,L),N=1,NRANG2) !a.u. to Ryd
        eigmax(L) = EIGENS(NRANG2,L)
        READ (ifile) (ENDS(N,L),N=1,NRANG2)
        WRITE(jfile) (ENDS(N,L),N=1,NRANG2)
        WRITE(jfile2) (ENDS(N,L),N=1,NRANG2)
        READ (ifile) MBUT(L)
        READ (ifile) (EBUT(I,L),I=1,MBUT(L))
        READ (ifile) (CBUT(I,L),I=1,MBUT(L))
      ENDDO

      READ (ifile) NAST
      if(NAST.gt.MZTAR)then
        print *,'STOP! Increase dimension MZTAR to:',NAST
        stop
      endif
      READ (ifile) (JAT(I),I=1,NAST)
      READ (ifile) (ENAT(I),I=1,NAST)

c Print target states

      if( iprint.ge.1 ) then
        write(iout,*) 'NELC,NZ,LRANG1,LRANG2,NRANG2,LAMAX,RA,NAST:',
     *          NELC,NZ,LRANG1,LRANG2,NRANG2,LAMAX,RA,NAST
        write(iout,*) 'Target states:'
        do i = 1,nast
          LPTY(i) = 0
          if( JAT(I).lt.0 ) LPTY(i) = 1
          JAT(i) = abs(JAT(i))-1
          aj = 0.5*JAT(i)
          if( LPTY(i).eq.1 ) aj = -aj
          ee = 2.0*(enat(i)-enat(1))
          if( i.eq.1 ) write(iout,1001) aj,enat(1)
          if( i.gt.1 ) write(iout,1002) aj,ee,ee*13.6057
        enddo
 1001   format(f5.1,'    0.0',1pe17.9,' au')
 1002   format(f5.1,2f12.6,' eV')
      endif

C  FIT BUTTLE CORRECTION AND STORE FIT PARAMETERS IN  COEFF(I,L),I=1,3

      IF (KCMIN.GT.1) THEN
        DO L = 1,KCMIN-1
          COEFF(1,L) = zero
          COEFF(2,L) = zero
          COEFF(3,L) = zero
        ENDDO
      ENDIF

      IF (BSTO.NE.ZERO .OR. MJS.EQ.0) GOTO 90

C  IF MJS=1 and BSTO=0 USE IMPROVED FITTING PROCEDURE 
c  (M J SEATON, J.PHYS.B20(1987)L69-72) - remember to convert to Ryd!

      COEFF(3,1)=-30000.      !FLAG NEWBUT CASE KCMIN.GT.1
      DO L = KCMIN,KCMAX
        imax = MBUT(L)
        do i=1,imax
          e(i) = 2*EBUT(i,L)  !BECAUSE BUTFIT REDEFINES E
          f(i) = CBUT(i,L)    !UNNECESSARY AS F NOT REDEFINED
        enddo
        emax = 2*eigmax(L)
        CALL BUTFIT(imax,e,f,RA,emax,ALPHA,BETA,nNBUT,DELTA)
C        if(iprint.gt.1)write(iout,*)'Buttle',L,nNBUT,ALPHA,BETA
        IF (nNBUT.le.0) THEN
      print*,'NEW BUTFIT DIVERGING, USE SUBROUTINE INTBUT IN STGF/B'
      print*,'Ensure DBUT.DAT file is present when running outer region'
          goto 90
        ENDIF
        COEFF(1,L) = ALPHA
        COEFF(2,L) = BETA
        COEFF(3,L) = -10000*nNBUT
      ENDDO
      goto 120

C  IF MJS=0 OR IF BUTFIT DIVERGING, dump Buttle corrections into DBUT.DAT 
C  (STGF CANNOT HANDLE A MIXTURE OF OLD AND NEW BUTTLE FITS): keep as a.u.

        write(0,*)'buttle failed .... needs a parallel fix'
   90 OPEN(31,FILE='DBUT.DAT',FORM='UNFORMATTED')
C
      IF (KCMIN.GT.1) THEN
        DO L = 1,KCMIN-1
          MBUT(L) = 1
          EBUT(1,L) = zero
          CBUT(1,L) = zero
          eigmax(L) = zero
        ENDDO
      ENDIF
C
      DO L = 1, LRANG2
        COEFF(1,L) = zero
        COEFF(2,L) = zero
        COEFF(3,L) = zero
        WRITE (31) MBUT(L)
        WRITE (31) (EBUT(N,L),N=1,MBUT(L))
        WRITE (31) (CBUT(N,L),N=1,MBUT(L))
      ENDDO
c 1031 format(1p100e20.12)
      if(iprint.ge.0)write(iout,*) 
     *  'Note, Buttle correction for INTBUT routine put in DBUT.DAT'
  120 continue

c Finish writing header
      NIX=0                                       !TO SKIP READ OF WERNERS'
C      if(nproc.eq.1)then
        WRITE(jfile) RA,BSTO,HINT,eta,eta,NIX
        WRITE(jfile) ((COEFF(I,L),I=1,3),L=1,LRANG2)
        WRITE(jfile) NAST
        WRITE(jfile) (ENAT(I),I=1,NAST), (JAT(I),I=1,NAST),
     *               (isat,I=1,NAST), (LPTY(I),I=1,NAST)
        WRITE(jfile2) RA,BSTO,HINT,eta,eta,NIX
        WRITE(jfile2) ((COEFF(I,L),I=1,3),L=1,LRANG2)
        WRITE(jfile2) NAST
        WRITE(jfile2) (ENAT(I),I=1,NAST), (JAT(I),I=1,NAST),
     *               (isat,I=1,NAST), (LPTY(I),I=1,NAST)
C      endif

      IPOLPH = 2 
      if(iprint.ge.1)write(iout,*)'N+1 electron symmetries:'
c-----------------------------------------------------------------------
c  Loop over each total symmetry:
c      iam=-1

      ncfmax=0

      do loop = 1, INAST

        READ(ifile,end=99) J2P1,NPTY,NCFGP

        ncfmax=max(ncfmax,ncfgp)

c  The following IF structure allows automatic detection of where the
c  dipole matrices start: it is only used if you have set INAST too big.

        if( IPOLPH.eq.2 .and. J2P1.ge.0 ) then
          if(iprint.ge.1)write(iout,*)J2P1,NPTY,NCFGP
          backspace ifile        
          go to 10
        endif

        if( NCFGP.gt.MZNC2 ) then
          print*,'STOP! Increase dimension MZNC2 to:', NCFGP
          stop
        endif

        READ(ifile) MNP1,NCHAN
       if(iprint.ge.1)write(iout,*)loop,') J2P,NPTY,NCFGP,MNP1,NCHAN:',
     *                             J2P1-1,NPTY,NCFGP,MNP1,NCHAN
        if( NCHAN.gt.MZCHF ) then
          print*,'STOP! Increase dimension MZCHF to:', NCHAN
          stop
        endif

C   CPB store data for  sizeBP.dat for pstg3r allocations
        bpv1(loop)=NCHAN
        bpv2(loop)=MNP1-NCFGP
        bpv3(loop)=MNP1
        bpv4(loop)=abs(J2P1)-1
        bpv5(loop)=NPTY

c Initialize ksym pointer to symmetric hamilitonian matrix elements

        kcc = 0
        do i = 1, NCHAN
          do j = 1,i
            kcc = kcc + 1
            ksym(i,j) = kcc
            ksym(j,i) = kcc
          enddo
        enddo

        READ(ifile) (NCONAT(I),I=1,NAST)
        READ(ifile) (K2P(I),I=1,NCHAN)          !kappa

        MORE   = 1
        if( loop.eq.INAST ) MORE = 0            !case INAST/nproc.ne.integer
        if(mod(loop,nrec).eq.0)MORE=0

        if (J2P1.lt.0) then
          IPOLPH = 2
          J2P1 = -J2P1
        endif
        do i=1,NCHAN                            !NRB
          L2P(I) = 2*K2P(I)                     !2*kappa
          if(K2P(I).lt.0) L2P(I) = -L2P(I)-1    !K, analogue of L
          L2P(I)=L2P(I)-1                       !K-1
        enddo
        if(iprint.gt.1)write(iout,*) 'L2P:',(L2P(i),i=1,NCHAN)

        ILSPI(1,loop) = J2P1*NPTY
        ILSPI(2,loop) = MNP1
        ILSPI(3,loop) = NCHAN
        if( NPTY.gt.0 ) NPTY = 0
        if( NPTY.lt.0 ) NPTY = 1

        if(mod(loop,2).eq.1)then

        WRITE(jfile) J2P1-1,NSPN,NPTY,NCFGP,IPOLPH
        WRITE(jfile) MNP1,NRANG2*NCHAN,NCHAN
        WRITE(jfile) (NCONAT(I),I=1,NAST)
        WRITE(jfile) (L2P(I),I=1,NCHAN),(K2P(I),I=1,NCHAN) !kappa not used
        WRITE(jfile) MORE

        else

        WRITE(jfile2) J2P1-1,NSPN,NPTY,NCFGP,IPOLPH
        WRITE(jfile2) MNP1,NRANG2*NCHAN,NCHAN
        WRITE(jfile2) (NCONAT(I),I=1,NAST)
        WRITE(jfile2) (L2P(I),I=1,NCHAN),(K2P(I),I=1,NCHAN) !kappa not used
        WRITE(jfile2) MORE

        endif

c Asymptotic coeffs

        READ (ifile) IKMAX
        IF (IKMAX.GT.0) THEN
          DO LAM = 1,IKMAX
            READ (ifile) ((CF(J,I,LAM),J=1,NCHAN),I=1,NCHAN)
          ENDDO
        ENDIF
        IF(IKMAX.LT.LAMAX)THEN
          DO LAM=IKMAX+1,LAMAX
            DO I=1,NCHAN
              DO J=I,NCHAN
                CF(J,I,LAM)=ZERO
              ENDDO
            ENDDO
          ENDDO
        ENDIF

c  Hamiltonian matrix

C  Continuum-continuum blocks

C
        kccc=((NCHAN*(NCHAN+1)))/2
        allocate(wrkcc(nrang2,nrang2,kccc))
C
        DO ICH = 1,NCHAN
          DO JCH = ICH,NCHAN
            READ (ifile) I,J
            kcc = ksym(I,J)
            IF (I.EQ.J) THEN
              READ (ifile) ((wrkcc(M,N,kcc),N=M,NRANG2),M=1,NRANG2)
c  Note, the diagonal block needs to be symmetrised
              do M=1,NRANG2
                do N=M,NRANG2
                  wrkcc(N,M,kcc)=wrkcc(M,N,kcc)
                enddo
              enddo
c              print*,kcc,i,j,ich,jch,(wrkcc(M,M,kcc),M=1,NRANG2)
            ELSE
              READ (ifile) ((wrkcc(M,N,kcc),N=1,NRANG2),M=1,NRANG2)
            ENDIF
          ENDDO
        ENDDO
c  Note, transpose is written out, because input is on one side of the
c  triangle (JCH = ICH,NCHAN) and output is on other side (JCH = 1,ICH)
        DO ICH = 1,NCHAN
          DO JCH = 1,ICH
            kcc = ksym(JCH,ICH)
           if(mod(loop,2).eq.1)then
            WRITE(jfile) ((wrkcc(N,M,kcc),N=1,NRANG2),M=1,NRANG2)
           else
            WRITE(jfile2) ((wrkcc(N,M,kcc),N=1,NRANG2),M=1,NRANG2)
           endif
          ENDDO
        ENDDO

       deallocate(wrkcc)
       call comm_barrier()

        IF (NCFGP.GT.0) THEN

C  Continuum-bound blocks

          allocate(wrkcb(NRANG2,NCFGP,NCHAN))       

          DO ICH = 1,NCHAN
            READ (ifile) I,((wrkcb(M,N,I),N=1,NCFGP),M=1,NRANG2)
          ENDDO
          DO ICH = 1,NCHAN
            if(mod(loop,2).eq.1)then
            WRITE(jfile) ((wrkcb(M,N,ICH),N=1,NCFGP),M=1,NRANG2)
            else
            WRITE(jfile2)((wrkcb(M,N,ICH),N=1,NCFGP),M=1,NRANG2)
            endif
          ENDDO

          deallocate(wrkcb)

C  Bound-bound elements

          allocate(wrkbb(NCFGP))

          DO M = 1,NCFGP
            READ (ifile) (wrkbb(N),N=M,NCFGP)
            if(mod(loop,2).eq.1)then
            WRITE(jfile) (wrkbb(N),N=M,NCFGP)
            else
            WRITE(jfile2) (wrkbb(N),N=M,NCFGP)
            endif
          ENDDO

           deallocate(wrkbb)

        ENDIF


c  Asymptotic coefficients

        if( LAMAX.GT.0 ) then
          do I = 1, NCHAN
            if(mod(loop,2).eq.1)then
            WRITE(jfile) ((2*CF(J,I,K),K=1,LAMAX),J=I,NCHAN)  !2* NRB
            else
            WRITE(jfile2) ((2*CF(J,I,K),K=1,LAMAX),J=I,NCHAN)  !2* NRB
            endif
          enddo
        endif

        INAST2 = loop
      enddo

c-----------------------------------------------------------------------
c  Dipole matrix


      if(irecuph.eq.0)then
      close(jfile,status='delete')
      close(jfile2,status='delete')
      endif
c     
c  gather data for sizeBP.dat
      allocate(bp1(2*nproc))
      allocate(bp2(2*nproc))
      allocate(bp3(2*nproc))
      allocate(bp4(2*nproc))
      allocate(bp5(2*nproc))
      call comm_gather(bpv1,2,bp1,2)
      call comm_gather(bpv2,2,bp2,2)
      call comm_gather(bpv3,2,bp3,2)
      call comm_gather(bpv4,2,bp4,2)
      call comm_gather(bpv5,2,bp5,2)
c
c   Write sizeBP.dat file
c
      if(iam.eq.0)then
      OPEN(UNIT=765,STATUS='UNKNOWN',FORM='FORMATTED',FILE='sizeBP.dat')
      do II=1,2*nproc,1
       NCHAN=bp1(II)
       MNP1W=bp2(II)
       MNP1=bp3(II)
       J2=bp4(II)
       JPI=bp5(II)
       IF(JPI.eq.1)JPI=0
       IF(JPI.eq.-1)JPI=1
       write(765,995)NCHAN,MNP1W,MNP1,
     A            ' 2J= ',J2,' PI ',JPI
       enddo
      endif
 995      FORMAT(3I7,A5,I7,A4,I7)
      if((irecupd.ne.1).and.(mod(loop,2).eq.0))goto 99

      if( IPOLPH.le.1 ) goto 99
   10 continue
c
      deallocate(bp1)
      deallocate(bp2)
      deallocate(bp3)
      deallocate(bp4)
      deallocate(bp5)
c-----------------------------------------------------------------------
c  Dipole matrix
c       
c      PARAMETER (INML=MZNR2/MZNC2, JNML=MZNC2/MZNR2, KNML=INML+JNML)
c      PARAMETER (ND24=MZNR2*INML/KNML+MZNC2*JNML/KNML)

       inml=nrang2/ncfmax
       jnml=ncfmax/nrang2
       knml=inml+jnml
       nd24=nrang2*inml/knml+ncfmax*jnml/knml

      allocate(wrkd(nd24,nd24,2))
c
c    To be implemented...
c      if(nproc.gt.1)then
c        print *,'Using a single RECUPD.DAT dipole file....'
c        print *,'Re-run with 1 RECUPH.DAT file if necessary.'
c      endif
c
c  Open dipole matrix output file
      i1=iam/100
      i2=(iam-100*(iam/100))/10
      i3=iam-(100*(iam/100))-i2*10
      recd='RECUPD'//NUM(i1)//NUM(i2)//NUM(i3) 
      OPEN( UNIT=kfile, STATUS='UNKNOWN', FORM='UNFORMATTED',
     *      FILE=recd)

c Loop over pairs of symmetries for dipole

      do loop2 = 2, INAST2
       do loop1 = 1,loop2-1
        MJ    = ILSPI(1,loop1)
        MMNP2 = ILSPI(2,loop1)
        MCHAN = ILSPI(3,loop1)
        NJ    = ILSPI(1,loop2)
        MNP2  = ILSPI(2,loop2)
        NCHAN = ILSPI(3,loop2)
        mcfg  = MMNP2-MCHAN*NRANG2
        ncfg  = MNP2 -NCHAN*NRANG2
        jdif  = (abs(MJ)-abs(NJ))/2
        if( MJ*NJ.lt.0 .and. abs(jdif).le.1 ) then
          if(iprint.ge.1)write(iout,*) 'Dipole transition 2J+1:',mj,nj

c Selection rules ok.  read/write dipole matrix elements

          I = -2
          J = -2

c Continuum-continuum dipole

          do nch = 1, NCHAN
            m2 = 0
            do mch = 1, MCHAN
              if( I.eq.-2 .and. J.eq.-2 ) READ (ifile,end=98) I,J,K,L
c              print*,i,j,k,l
              IF (I.EQ.-1 .AND. J.EQ.-1) GOTO 20
              if( I.eq.nch .and. J.eq.mch ) then
                I = -2
                J = -2
                READ (ifile) ((wrkd(M,N,1),M=1,K),N=1,L)
                READ (ifile) ((wrkd(M,N,2),M=1,K),N=1,L)
                WRITE(kfile) ((wrkd(M,N,1),M=1,K),N=1,L)
                WRITE(kfile) ((wrkd(M,N,2),M=1,K),N=1,L)
              else
                WRITE(kfile) (zero,M=1,NRANG2*NRANG2)
                WRITE(kfile) (zero,M=1,NRANG2*NRANG2)
              endif
              m2 = m2 + NRANG2
            enddo
            if( m2.lt.MMNP2 ) then
              if( I.eq.-2 .and. J.eq.-2 ) READ (ifile,end=98) I,J,K,L
c              print*,i,j,k,l
              IF (I.EQ.-1 .AND. J.EQ.-1) GOTO 20
              if( I.eq.nch .and. J.eq.0 ) then
                if( mcfg.ne.L ) then
              write(iout,*) 'STOP: mcfg and L must be the same:',mcfg,L
                  stop
                endif
                I = -2
                J = -2
                READ (ifile) ((wrkd(M,N,1),M=1,K),N=1,L)
                READ (ifile) ((wrkd(M,N,2),M=1,K),N=1,L)
                WRITE(kfile) ((wrkd(M,N,1),M=1,K),N=1,L)
                WRITE(kfile) ((wrkd(M,N,2),M=1,K),N=1,L)
              else
                WRITE(kfile) (zero,M=1,NRANG2*mcfg)
                WRITE(kfile) (zero,M=1,NRANG2*mcfg)
              endif
            endif
            WRITE(kfile) ((ABUTL,M=1,NRANG2),N=1,MCHAN)
            WRITE(kfile) ((ABUTV,M=1,NRANG2),N=1,MCHAN)
          enddo
 
c Bound-continuum dipole

          if( ncfg.gt.0 ) then
            do mch = 1, MCHAN
              if( I.eq.-2 .and. J.eq.-2 ) READ (ifile,end=98) I,J,K,L
c              print*,i,j,k,l
              IF (I.EQ.-1 .AND. J.EQ.-1) GOTO 20
              if( I.eq.0 .and. J.eq.mch ) then
                if( ncfg.ne.K ) then
                  print *,'STOP: ncfg and K must be the same:',ncfg,K
                  stop
                endif
                I = -2
                J = -2
                READ (ifile) ((wrkd(M,N,1),M=1,K),N=1,L)
                READ (ifile) ((wrkd(M,N,2),M=1,K),N=1,L)
                WRITE(kfile) ((wrkd(M,N,1),M=1,K),N=1,L)
                WRITE(kfile) ((wrkd(M,N,2),M=1,K),N=1,L)
              else
                WRITE(kfile) (zero,M=1,ncfg*NRANG2)
                WRITE(kfile) (zero,M=1,ncfg*NRANG2)
              endif
            enddo

c Bound-bound dipole

            if( mcfg.gt.0 ) then
              if( I.eq.-2 .and. J.eq.-2 ) READ (ifile,end=98) I,J,K,L
c              print*,i,j,k,l
              IF (I.EQ.-1 .AND. J.EQ.-1) GOTO 20
              if( I.eq.0 .and. J.eq.0 ) then
                if( mcfg.ne.L .or. ncfg.ne.K ) then
                  print *,'STOP: mcfg,ncfg and L,K=',mcfg,L,ncfg,K
                  stop
                endif
                I = -2
                J = -2
                READ (ifile) ((wrkd(M,N,1),M=1,K),N=1,L)
                READ (ifile) ((wrkd(M,N,2),M=1,K),N=1,L)
              else
                do N=1,mcfg
                  do M=1,ncfg
                    wrkd(M,N,1)=zero
                    wrkd(M,N,2)=zero
                  enddo
                enddo
              endif
              NDIMEN = NRANG2
              NTIMES = (ncfg-1)/NDIMEN + 1
              n2 = 0
              do nt = 1,NTIMES
                n22 = min(n2+NDIMEN,ncfg)
                WRITE(kfile) ((wrkd(M,N,1),M=n2+1,n22),N=1,mcfg)
                WRITE(kfile) ((wrkd(M,N,2),M=n2+1,n22),N=1,mcfg)
                n2 = n22
              enddo
            endif
          endif

c Write zeros for dipole matrix Buttle correction

   20     READ (ifile,iostat=ioio) I,J,K,L
          if(ioio.ne.0)goto 98
c          print*,i,j,k,l
          if( I.ne.-1 .and. J.ne.-1 .and. K.ne.-1 .and. L.ne.-1 ) then
            print*,'WARNING: dipole matrices not finished off:',i,j,k,l
          endif
          n2 = NCHAN*NRANG2
          if( n2.LT.MNP2 ) then
            WRITE(kfile) ((ABUTL,J=1,MNP2-n2),I=1,MCHAN)
            WRITE(kfile) ((ABUTV,J=1,MNP2-n2),I=1,MCHAN)
          endif
          do MCH = 1,MCHAN
            WRITE(kfile) ((BBUTL,J=1,NCHAN),I=1,NRANG2)
            WRITE(kfile) ((BBUTV,J=1,NCHAN),I=1,NRANG2)
          enddo
          m2 = MCHAN*NRANG2
          if( m2.LT.MMNP2 ) then
            WRITE(kfile) ((BBUTL,J=1,NCHAN),I=1,MMNP2-m2)
            WRITE(kfile) ((BBUTV,J=1,NCHAN),I=1,MMNP2-m2)
          endif
          WRITE(kfile) ((BBUTL,J=1,NCHAN),I=1,MCHAN)
          WRITE(kfile) ((BBUTV,J=1,NCHAN),I=1,MCHAN)

c CLEBSCH GORDAN COEFFICIENTS FOR THE POLARIZABILITY CALC

          READ (ifile) MAXM1
          if(MAXM1.gt.MZLR2)then
            print *,'STOP! Increase dimension MZLR2 to:',MAXM1
            stop
          endif

          READ (ifile) (temp(M),M=1,MAXM1)
          WRITE(kfile) MAXM1,(temp(M),M=1,MAXM1)

c ANGULAR INTEGRALS NEEDED FOR OUTER REGION INTEGRATION

          if(outer) then
            READ (ifile,end=96) ((AC(I,J),J=1,MCHAN),I=1,NCHAN)
            READ (ifile) ((BLC(I,J),J=1,MCHAN),I=1,NCHAN)
            READ (ifile) ((BVC(I,J),J=1,MCHAN),I=1,NCHAN)
        if(iprint.ge.1)write(iout,*) 'outer region contributions found'
            go to 97
          endif
   96 write(iout,*)'outer region contributions not found:zeros inserted'
          do j=1,MZCHF
            do i=1,MZCHF
              AC(i,j)  = zero
              BLC(i,j) = zero
              BVC(i,j) = zero
            enddo
          enddo
   97     WRITE(kfile) ((AC (I,J),J=1,MCHAN),I=1,NCHAN)
          WRITE(kfile) ((BLC(I,J),J=1,MCHAN),I=1,NCHAN)
          WRITE(kfile) ((BVC(I,J),J=1,MCHAN),I=1,NCHAN)
        endif
       enddo
      enddo

   98 if(iprint.ge.1)write(iout,*) 'RECUPDXXX created'
      close(kfile)
      deallocate(wrkd)
   99 continue 

      if(irecuph.ne.0)then
      close(jfile)
      close(jfile2)
      endif

      if(idelete.ne.0)then
       close(ifile,status='delete')
      else
       close(ifile)
      endif

C      stop 'Normal End'
      end
C      

C-----------------------------------------------------------------------

      SUBROUTINE BUTFIT(IMAX,E,F,RA,EMAX,ALPHA,BETA,NBUT,DELTA)
      IMPLICIT REAL*8 (A-H,O-Z)
C
C  FITTING OF BUTTLE CORRECTIONS -- M J SEATON, J.PHYS.B20(1987)L69-72.
C
C  IMAX = NUMBER OF POINTS FOR WHICH CORRECTION CALCULATED.
C  E(I) = ENERGY POINTS
C  F(I) = BUTTLE CORRECTION
C  RA = BOUNDARY RADIUS
C  EMAX = EIGENS(NRANG2,L)
C  ALPHA, BETA = FIT PARAMETERS, NBUT =...
C  DELTA = ACCURACY ACHIEVED
C
      LOGICAL POLE
C
      DIMENSION E(IMAX),F(IMAX)
C-----------------------------------------------------------------------
C
C  INITIALISATIONS
C
      D = RA*RA
      DO 10 I = 1,IMAX
        E(I) = E(I)*D
   10 CONTINUE
      T = 0.5D0
      IF(EMAX.GT.0.0D0)T=T + RA*SQRT(EMAX)/3.141592654D0
      NBUT = T
      DELTA = 1.0D30
      ALPHA = 1.0D0
      BETA = 0.0D0
C
C START ITERATIONS FOR FIT
C
      DO 50 KK = 1,15
C
        X11 = 0.0D0
        X12 = 0.0D0
        X22 = 0.0D0
        Y1 = 0.0D0
        Y2 = 0.0D0
        DELTA0 = DELTA
        DELTA = 0.0D0
C
C    START SUM OVER POINTS I
C
        DO 40 I = 1,IMAX
          U = BETA + E(I)
C
C    CALCULATE FUNCTIONS B(U) AND C(U)
C
          B = 0.0D0
          C = 0.0D0
C    CASE OF U.GT.0.04
          IF (U.GT.0.04D0) THEN
            FK = SQRT(U)
            POLE = .FALSE.
            G = -1.5707963D0
            DO 20 N = 0,NBUT
              G = G + 3.141592654D0
              IF (ABS(FK-G).GT.0.3D0) THEN
                A = 1.0D0/ (U-G*G)
                B = B + A
                C = C - A*A
C
              ELSE
                POLE = .TRUE.
                D1 = FK - G
              ENDIF
C
   20       CONTINUE
            IF (POLE) THEN
              D2 = D1*D1
              D = 0.33333333D0*
     A            D1* (1.0D0+0.066666667D0*
     B            D2* (1.0D0+0.0952381D0*D2))
              A = 1.0D0/ (2.0D0*FK-D1)
              BB = (D+A)/FK
              D = 0.33333333D0* (1.0D0+
     A            D2* (0.2D0+0.031746032D0*D2))
              C = 2.0D0*C + 0.5D0* (D-A*A-BB)/U
              B = 2.0D0*B + BB
C
            ELSE
              T = TAN(FK)
              TK = T/FK
              B = 2.0D0*B + TK
              C = 2.0D0*C + 0.5D0*
     A            (1.0D0+T*T-TK)/U
            ENDIF
C
C    SUM FOR U.LE..04
          ELSE
            G = -1.5707963D0
            DO 30 N = 0,NBUT
              G = G + 3.141592654D0
              A = 1.0D0/ (U-G*G)
              B = B + A
              C = C - A*A
   30       CONTINUE
C
C    CASE OF U.LT..04 AND U.GT.-.04
            IF (U.GT.-0.04D0) THEN
              B = (0.4D0*U+1.0D0)
     A            *U*0.33333333D0
     A            + B*2.0D0 + 1.0D0
              C = ((0.48571429D0*U+0.8D0)*U
     A            +1.0D0)*0.33333333D0
     B            + C*2.0D0
C
C    CASE OF U.LT.-.04
            ELSE
              FK = SQRT(-U)
              T = TANH(FK)
              TK = T/FK
              B = 2.0D0*B + TK
              C = 2.0D0*C + 0.5D0
     A            * (1.0D0-T*T-TK)/U
            ENDIF
C
          ENDIF
C
C    INCREMENT MATRICES X AND Y
          DF = F(I) - ALPHA*B
          DD = ABS(DF)
          IF (DELTA.LT.DD) DELTA = DD
          X11 = X11 + B*B
          X12 = X12 + B*C
          X22 = X22 + C*C
          Y1 = Y1 + DF*B
          Y2 = Y2 + DF*C
C
   40   CONTINUE
C
C  SOLVE EQUATIONS AND INCREMENT ALPHA AND BETA
C
C+++ MODIFICATIONS MADE BY MJS, 22.12.86.
CJZ    X12=ALPHA*X12
        DET = 1.0D0/ (X11*X22-X12*X12)
CJZ    BETA=BETA+DET*(-X12*Y1+X11*Y2)
        BETA = BETA + DET* (-X12*Y1+X11*Y2)/ALPHA
C+++ END MODIFICATIONS
        ALPHA = (X22*Y1-X12*Y2)*DET + ALPHA
C
C    CHECK CONVERGENCE
C
        IF (DELTA.LT.2.0D-4) GOTO 60
C
C  END ITERATIONS
C
   50 CONTINUE
      NBUT = -NBUT
      IF (DELTA.GT.DELTA0) NBUT = 0
C     AS PROCEDURE NOT CONVERGED OR DISTINCTLY DIVERGING - WE'88NOV17.
C
   60 CONTINUE
C
      END

C-----------------------------------------------------------------------

      SUBROUTINE HEAD(jfile)
      IMPLICIT REAL*8 (A-H,O-Z)

      INCLUDE 'PARAM'

      COMMON/COM1/ENDS(MZNR2,MZLR2),ENAT(MZTAR),
     *            EIGENS(MZNR2,MZLR2),COEFF(3,MZLR2)
      COMMON/COM2/JAT(MZTAR),LPTY(MZTAR), MAXNLG(MZLR2)
      COMMON/COM3/NELC,NZ,LRANG1,LRANG2,NRANG2,LAMAX,ICODE,LAM,IZESP,
     *            JRELOP,maxnc,NAST,isat,NIX,RA,BSTO,HINT,eta
C
c Write header to standard RECUPH.DAT hamilitonian output file
C
      WRITE(jfile) NELC,NZ,LRANG1,LRANG2,NRANG2,LAMAX,ICODE,LAM,IZESP,
     *             JRELOP,JRELOP,JRELOP
      WRITE(jfile) (MAXNLG(L),L=1,LRANG1), (MAXNLG(L),L=1,LRANG1),
     *             (maxnc,L=1,LRANG1)
C
      DO L = 1,LRANG2
        WRITE(jfile) (2*EIGENS(N,L),N=1,NRANG2)     !a.u. to Ryd - nrb
        WRITE(jfile) (ENDS(N,L),N=1,NRANG2)
      ENDDO
C
      WRITE(jfile) RA,BSTO,HINT,eta,eta,NIX
      WRITE(jfile) ((COEFF(I,L),I=1,3),L=1,LRANG2)
      WRITE(jfile) NAST
      WRITE(jfile) (ENAT(I),I=1,NAST), (JAT(I),I=1,NAST),
     *             (isat,I=1,NAST), (LPTY(I),I=1,NAST)
C
      RETURN
      END

c-----------------------------------------------------------------------
      SUBROUTINE DTO3DIR(INAST,nproca,IFILE)
      use mod_namelist, only : IMEMDIR,IRECUPH,IRECUPD,IDELETE         
c
c
c  Converts DARC to RMBP file format: DSTG2.DAT to RECUPH.DAT and RECUPD.DAT.
c
c  The following need to be predefined:
c  INAST = number of N+1 electron symmetries on file
c  ifile = unit number for DSTG2.DAT input
c  nproc = number of RECUPH.DAT files to be written (=1 for serial codes).

      use comm_interface, only : iam,nproc,comm_barrier,comm_gather
      use param1 
      use pdam
      IMPLICIT REAL*8(A-H,O-Z)

c  DSTG2.DAT is the file from the DARC program dstg2.f, 
c  and can contain both Hamiltonian and dipole matrix elements.

c  The DTO3 routine reads this file and creates file RECUPH.DAT with
c  the Hamiltonians, and RECUPD.DAT with the dipoles if present,
c  in the format required by the BPRM program stg3r.f
c  (the format was defined by Berrington et al 1995 in CPC 92,290-420).

c  It also converts the Buttle correction to 'MJS' style.
c  It also detects any external-region contributions to the dipole matrix
c  (these can now be calculated in dstg2.f with some mods by kab).

c   DARC style Buttle correction written to DBUT.DAT (case MJS=0) for
c   automatic read by stgf.

c Include the dimension parameters MZLR2,MZNR2,MZNC2,MZLMX,MZCHF,MZSLP,MZTAR
c using the Badnell (stg3r.f etc) convention:
C
C      INCLUDE 'PARAM'
C
c This could be done using the DARC dstg2.f dimension plants as follows:
c      PARAMETER (MZLR2={NK})
c      PARAMETER (MZNR2={NB})
c      PARAMETER (NXNC2={NC})
c      PARAMETER (MZCHF={CH})
c      PARAMETER (MZTAR={NL})
c      PARAMETER (MZLMX=4)
c      PARAMETER (MZSLP=100)

c Derive further parameters to set dimensions.
c The biggest array here is work, whose size=MZNR2*MZNC2*MZCHF*MZCHF/2.

c      PARAMETER (INML=MZNR2/MZNC2, JNML=MZNC2/MZNR2, KNML=INML+JNML)
c      PARAMETER (ND24=MZNR2*INML/KNML+MZNC2*JNML/KNML)
c      PARAMETER (kccc=(MZCHF*(MZCHF+1))/2)
      PARAMETER (NBUT=2*(MZNR2-1))
      parameter (zero=0.0)

      CHARACTER IHED*80,RECORD*20
      CHARACTER*1 NUM(0:9)
      CHARACTER*9 REC,RECD
      CHARACTER*10 SCRAT

      logical outer,bnew

      DIMENSION MAXNQN(MZLR2), MINNQN(MZLR2),
     *          MAXFUL(MZLR2), LAG(MZLR2), KAG(MZLR2), JAG(MZLR2),
     *          NCONAT(MZTAR), K2P(MZCHF), L2P(MZCHF),
     *          MBUT(MZLR2), ksym(MZCHF,MZCHF),
     *          ILSPI(3,MZSLP), temp(MZLR2)
      DIMENSION AC(MZCHF,MZCHF), BLC(MZCHF,MZCHF), BVC(MZCHF,MZCHF),
     *          EBUT(NBUT,MZLR2),CBUT(NBUT,MZLR2),CF(MZCHF,MZCHF,MZLMX),
     *          e(NBUT), f(NBUT), eigmax(MZLR2)
C     *          wrkbb(MZNC2)
C
       real*8,allocatable :: wrkcc(:,:,:),wrkcb(:,:,:)
       real*8,allocatable :: wrkd(:,:,:),wrkbb(:)
       INTEGER :: bpv1(2),bpv2(2),bpv3(2),bpv4(2),bpv5(2)
c      real*8,allocatable :: arrayda(:),matda(:,:)
      integer,allocatable :: bp1(:),bp2(:),bp3(:),bp4(:),bp5(:)
      COMMON/COM1/ENDS(MZNR2,MZLR2),ENAT(MZTAR),
     *            EIGENS(MZNR2,MZLR2),COEFF(3,MZLR2)
      COMMON/COM2/JAT(MZTAR),LPTY(MZTAR), MAXNLG(MZLR2)
      COMMON/COM3/NELC,NZ,LRANG1,LRANG2,NRANG2,LAMAX,ICODE,LAM,IZESP,
     *            JRELOP,maxnc,NAST,isat,NIX,RA,BSTO,HINT,eta

c  MJS   = 1 (default) for producing Buttle fit;
c        = 0 for dump on DBUT.DAT (this also occurs if fit fails).
c  outer = true (default) for outer region contributions, if any;
c        = false for inserting zeros.
c  jfile = 32 (default) unit number for RECUPD.DAT output.
c  kfile = 33 (default) unit number for RECUPH.DAT output.
c  iprint= 1 (default) for printing useful stuff on standard output;
c        = 0 for emergency printing only.
c  The other parameters set here should be acceptable for BPRM.

c      data MJS/1/, outer/.TRUE./, jfile/32/, kfile/33/, iprint/1/
c      data ICODE/25/, IZESP/0/, JRELOP/1/, maxnc/0/, isat/0/, NSPN/0/
c      data ABUTL/0.0/, ABUTV/0.0/, BBUTL/0.0/, BBUTV/0.0/, eta/0.0/
      DATA NUM/'0','1','2','3','4','5','6','7','8','9'/
      DATA iout/300/
c
      MJS=1
      outer=.TRUE.
      jfile2=29
      ifile=30
      jfile=32
      kfile=33
      iprint=1
      ICODE=25
      JRELOP=1
      IZESP=0
      maxnc=0
      isat=0
      NSPN=0
      ABUTL=0.0d0
      ABUTV=0.0d0
      BBUTL=0.0d0
      BBUTV=0.0d0
      eta=0.0d0
c
c  Check dimensions for number of symmetries input

      if( INAST.gt.MZSLP ) then
        print*,'STOP! increase dimension MZLSP to:',MZSLP
        stop
      endif

c
c nrec=number of symmetries per file
c
C      nrec=inast/nproc
      nrec=2                 !pair of dipoles per file
C      if(nrec.le.0)nrec=1
C
      iam1=2*iam
      iam2=iam1+1
C
      i1=iam1/100
      i2=(iam1-100*(iam1/100))/10
      i3=iam1-(100*(iam1/100))-i2*10
      REC='RECUPH'//NUM(i1)//NUM(i2)//NUM(i3)
c  Open Hamiltonian output file and info file
        OPEN( UNIT=jfile, STATUS='UNKNOWN', FORM='UNFORMATTED',
     *      FILE=REC,access='sequential',action='write')

      i1=iam2/100
      i2=(iam2-100*(iam2/100))/10
      i3=iam2-(100*(iam2/100))-i2*10
      REC='RECUPH'//NUM(i1)//NUM(i2)//NUM(i3)
c  Open Hamiltonian output file and info file
        OPEN( UNIT=jfile2, STATUS='UNKNOWN', FORM='UNFORMATTED',
     *      FILE=REC,access='sequential',action='write')
c-----------------------------------------------------------------------
c Read header from DSTG2.DAT file

c Some of the variables are DARC-specific and do not seem to correspond
c to anything in BPRM, eg speed of light CL, so we will try to ignore them.

c On the other hand, there are some variables that BPRM would like,
c which the DARC file does not have, eg. LAMAX, so we will try to derive them.

c Along the way, we will also check dimensions.

c      rewind ifile
      READ (ifile) IHED,RECORD
      if(iprint.ge.1)write(iout,*)IHED,RECORD
      READ (ifile) KBMAX,KCMIN,KCMAX,KX,NRANG2,NELC,NZ
      READ (ifile) LAMBB,LAMBC,LAMCC

      LRANG1 = KX
      LRANG2 = KCMAX                ! but starts at KCMIN
      LAMAX  = LAMBB
      LAM    = 1
      if( LAMBC.gt.0 ) LAM = 2
      if( LAMCC.gt.0 ) LAM = 3

      if( LRANG1.gt.MZLR2 .or. LRANG2.gt.MZLR2  ) then
        print*,'STOP! Increase dimension MZLR2 to:',max(LRANG1,LRANG2)
        stop
      endif
      if( LAMAX.gt.MZLMX ) then
        print*,'STOP! Increase dimension MZLMX to:',LAMAX
        stop
      endif
      if( NRANG2.gt.MZNR2 ) then
        print*,'STOP! Increase dimension MZNR2 to:',NRANG2
        stop
      endif

      IF( KBMAX.GT.0 ) THEN
        READ (ifile) (MINNQN(L),L=1,KX), (MAXNQN(L),L=1,KX),
     *               (MAXNLG(L),L=1,KX), (MAXFUL(L),L=1,KBMAX),
     *               (LAG(L),L=1,KX), (KAG(L),L=1,KX), (JAG(L),L=1,KX)
      ELSE
        READ (ifile) (MINNQN(L),L=1,KX), (MAXNQN(L),L=1,KX),
     *               (MAXNLG(L),L=1,KX), (MAXFUL(L),L=1,KX),
     *               (LAG(L),L=1,KX),(KAG(L),L=1,KX), (JAG(L),L=1,KX)
      ENDIF

      READ (ifile) RA,BSTO,HINT,CL,DR1
      READ (ifile) NIX
      READ (ifile) (IRX,I=1,NIX)
      READ (ifile) NUMORB

c Write header to standard RECUPH.DAT hamilitonian output file

        WRITE(jfile) NELC,NZ,LRANG1,LRANG2,NRANG2,LAMAX,ICODE,LAM,IZESP,
     *               JRELOP,JRELOP,JRELOP
        WRITE(jfile) (MAXNLG(L),L=1,LRANG1), (MAXNLG(L),L=1,LRANG1),
     *               (maxnc,L=1,LRANG1)
        WRITE(jfile2)NELC,NZ,LRANG1,LRANG2,NRANG2,LAMAX,ICODE,LAM,IZESP,
     *               JRELOP,JRELOP,JRELOP
        WRITE(jfile2) (MAXNLG(L),L=1,LRANG1), (MAXNLG(L),L=1,LRANG1),
     *               (maxnc,L=1,LRANG1)

      IF (KCMIN.GT.1) THEN
        DO L = 1,KCMIN-1
          DO N=1,NRANG2
            ENDS(N,L)=ZERO
            EIGENS(N,L)=ZERO
          ENDDO
            WRITE(jfile) (EIGENS(N,L),N=1,NRANG2)
            WRITE(jfile) (ENDS(N,L),N=1,NRANG2)
            WRITE(jfile2) (EIGENS(N,L),N=1,NRANG2)
            WRITE(jfile2) (ENDS(N,L),N=1,NRANG2)
        ENDDO
      ENDIF
c Finish reading header

      DO L = KCMIN,KCMAX
        READ (ifile) (EIGENS(N,L),N=1,NRANG2)
        WRITE(jfile) (2*EIGENS(N,L),N=1,NRANG2) !a.u. to Ryd
        WRITE(jfile2) (2*EIGENS(N,L),N=1,NRANG2) !a.u. to Ryd
        eigmax(L) = EIGENS(NRANG2,L)
        READ (ifile) (ENDS(N,L),N=1,NRANG2)
        WRITE(jfile) (ENDS(N,L),N=1,NRANG2)
        WRITE(jfile2) (ENDS(N,L),N=1,NRANG2)
        READ (ifile) MBUT(L)
        READ (ifile) (EBUT(I,L),I=1,MBUT(L))
        READ (ifile) (CBUT(I,L),I=1,MBUT(L))
      ENDDO

      READ (ifile) NAST
      if(NAST.gt.MZTAR)then
        print *,'STOP! Increase dimension MZTAR to:',NAST
        stop
      endif
      READ (ifile) (JAT(I),I=1,NAST)
      READ (ifile) (ENAT(I),I=1,NAST)

c Print target states

      if( iprint.ge.1 ) then
        write(iout,*) 'NELC,NZ,LRANG1,LRANG2,NRANG2,LAMAX,RA,NAST:',
     *          NELC,NZ,LRANG1,LRANG2,NRANG2,LAMAX,RA,NAST
        write(iout,*) 'Target states:'
        do i = 1,nast
          LPTY(i) = 0
          if( JAT(I).lt.0 ) LPTY(i) = 1
          JAT(i) = abs(JAT(i))-1
          aj = 0.5*JAT(i)
          if( LPTY(i).eq.1 ) aj = -aj
          ee = 2.0*(enat(i)-enat(1))
          if( i.eq.1 ) write(iout,1001) aj,enat(1)
          if( i.gt.1 ) write(iout,1002) aj,ee,ee*13.6057
        enddo
 1001   format(f5.1,'    0.0',1pe17.9,' au')
 1002   format(f5.1,2f12.6,' eV')
      endif

C  FIT BUTTLE CORRECTION AND STORE FIT PARAMETERS IN  COEFF(I,L),I=1,3

      IF (KCMIN.GT.1) THEN
        DO L = 1,KCMIN-1
          COEFF(1,L) = zero
          COEFF(2,L) = zero
          COEFF(3,L) = zero
        ENDDO
      ENDIF

      IF (BSTO.NE.ZERO .OR. MJS.EQ.0) GOTO 90

C  IF MJS=1 and BSTO=0 USE IMPROVED FITTING PROCEDURE 
c  (M J SEATON, J.PHYS.B20(1987)L69-72) - remember to convert to Ryd!

      COEFF(3,1)=-30000.      !FLAG NEWBUT CASE KCMIN.GT.1
      DO L = KCMIN,KCMAX
        imax = MBUT(L)
        do i=1,imax
          e(i) = 2*EBUT(i,L)  !BECAUSE BUTFIT REDEFINES E
          f(i) = CBUT(i,L)    !UNNECESSARY AS F NOT REDEFINED
        enddo
        emax = 2*eigmax(L)
        CALL BUTFIT(imax,e,f,RA,emax,ALPHA,BETA,nNBUT,DELTA)
        if(iprint.gt.1)write(iout,*)'Buttle',L,nNBUT,ALPHA,BETA
        IF (nNBUT.le.0) THEN
      print*,'NEW BUTFIT DIVERGING, COPY DBUT.DAT file to outer region'
          goto 90
        ENDIF
        COEFF(1,L) = ALPHA
        COEFF(2,L) = BETA
        COEFF(3,L) = -10000*nNBUT
      ENDDO
      goto 120

C  IF MJS=0 OR IF BUTFIT DIVERGING, dump Buttle corrections into DBUT.DAT 
C  (STGF CANNOT HANDLE A MIXTURE OF OLD AND NEW BUTTLE FITS): keep as a.u.

        write(0,*)'buttle failed .... needs a parallel fix'
   90 OPEN(31,FILE='DBUT.DAT',FORM='UNFORMATTED')
C
      IF (KCMIN.GT.1) THEN
        DO L = 1,KCMIN-1
          MBUT(L) = 1
          EBUT(1,L) = zero
          CBUT(1,L) = zero
          eigmax(L) = zero
        ENDDO
      ENDIF
C
      DO L = 1, LRANG2
        COEFF(1,L) = zero
        COEFF(2,L) = zero
        COEFF(3,L) = zero
        WRITE (31) MBUT(L)
        WRITE (31) (EBUT(N,L),N=1,MBUT(L))
        WRITE (31) (CBUT(N,L),N=1,MBUT(L))
      ENDDO
c 1031 format(1p100e20.12)
      if(iprint.ge.0)write(iout,*) 
     *  'Note, Buttle correction for INTBUT routine put in DBUT.DAT'
  120 continue

c Finish writing header
      NIX=0                                       !TO SKIP READ OF WERNERS'
C      if(nproc.eq.1)then
        WRITE(jfile) RA,BSTO,HINT,eta,eta,NIX
        WRITE(jfile) ((COEFF(I,L),I=1,3),L=1,LRANG2)
        WRITE(jfile) NAST
        WRITE(jfile) (ENAT(I),I=1,NAST), (JAT(I),I=1,NAST),
     *               (isat,I=1,NAST), (LPTY(I),I=1,NAST)
        WRITE(jfile2) RA,BSTO,HINT,eta,eta,NIX
        WRITE(jfile2) ((COEFF(I,L),I=1,3),L=1,LRANG2)
        WRITE(jfile2) NAST
        WRITE(jfile2) (ENAT(I),I=1,NAST), (JAT(I),I=1,NAST),
     *               (isat,I=1,NAST), (LPTY(I),I=1,NAST)
C      endif

      IPOLPH = 2 
      if(iprint.ge.1)write(iout,*)'N+1 electron symmetries:'
c-----------------------------------------------------------------------
c  Loop over each total symmetry:
c      iam=-1

      ncfmax=0

      do loop = 1, INAST

        READ(ifile,end=99) J2P1,NPTY,NCFGP

        ncfmax=max(ncfmax,ncfgp)

c  The following IF structure allows automatic detection of where the
c  dipole matrices start: it is only used if you have set INAST too big.

        if( IPOLPH.eq.2 .and. J2P1.ge.0 ) then
          if(iprint.ge.1)write(iout,*)J2P1,NPTY,NCFGP
          backspace ifile        
          go to 10
        endif

        if( NCFGP.gt.MZNC2 ) then
          print*,'STOP! Increase dimension MZNC2 to:', NCFGP
          stop
        endif

        READ(ifile) MNP1,NCHAN
       if(iprint.ge.1)write(iout,*)loop,') J2P,NPTY,NCFGP,MNP1,NCHAN:',
     *                             J2P1-1,NPTY,NCFGP,MNP1,NCHAN
        if( NCHAN.gt.MZCHF ) then
          print*,'STOP! Increase dimension MZCHF to:', NCHAN
          stop
        endif

C   CPB  store data for  sizeBP.dat for pstg3r allocations
        bpv1(loop)=NCHAN
        bpv2(loop)=MNP1-NCFGP
        bpv3(loop)=MNP1
        bpv4(loop)=abs(J2P1)-1
        bpv5(loop)=NPTY     

c Initialize ksym pointer to symmetric hamilitonian matrix elements

        kcc = 0
        do i = 1, NCHAN
          do j = 1,i
            kcc = kcc + 1
            ksym(i,j) = kcc
            ksym(j,i) = kcc
          enddo
        enddo

        READ(ifile) (NCONAT(I),I=1,NAST)
        READ(ifile) (K2P(I),I=1,NCHAN)          !kappa

        MORE   = 1
        if( loop.eq.INAST ) MORE = 0            !case INAST/nproc.ne.integer
        if(mod(loop,nrec).eq.0)MORE=0

        if (J2P1.lt.0) then
          IPOLPH = 2
          J2P1 = -J2P1
        endif
        do i=1,NCHAN                            !NRB
          L2P(I) = 2*K2P(I)                     !2*kappa
          if(K2P(I).lt.0) L2P(I) = -L2P(I)-1    !K, analogue of L
          L2P(I)=L2P(I)-1                       !K-1
        enddo
        if(iprint.gt.1)write(iout,*) 'L2P:',(L2P(i),i=1,NCHAN)

        ILSPI(1,loop) = J2P1*NPTY
        ILSPI(2,loop) = MNP1
        ILSPI(3,loop) = NCHAN
        if( NPTY.gt.0 ) NPTY = 0
        if( NPTY.lt.0 ) NPTY = 1

        if(mod(loop,2).eq.1)then

        WRITE(jfile) J2P1-1,NSPN,NPTY,NCFGP,IPOLPH
        WRITE(jfile) MNP1,NRANG2*NCHAN,NCHAN
        WRITE(jfile) (NCONAT(I),I=1,NAST)
        WRITE(jfile) (L2P(I),I=1,NCHAN),(K2P(I),I=1,NCHAN) !kappa not used
        WRITE(jfile) MORE

        else

        WRITE(jfile2) J2P1-1,NSPN,NPTY,NCFGP,IPOLPH
        WRITE(jfile2) MNP1,NRANG2*NCHAN,NCHAN
        WRITE(jfile2) (NCONAT(I),I=1,NAST)
        WRITE(jfile2) (L2P(I),I=1,NCHAN),(K2P(I),I=1,NCHAN) !kappa not used
        WRITE(jfile2) MORE

        endif

c Asymptotic coeffs

        READ (ifile) IKMAX
        IF (IKMAX.GT.0) THEN
          DO LAM = 1,IKMAX
            READ (ifile) ((CF(J,I,LAM),J=1,NCHAN),I=1,NCHAN)
          ENDDO
        ENDIF
        IF(IKMAX.LT.LAMAX)THEN
          DO LAM=IKMAX+1,LAMAX
            DO I=1,NCHAN
              DO J=I,NCHAN
                CF(J,I,LAM)=ZERO
              ENDDO
            ENDDO
          ENDDO
        ENDIF

c  Hamiltonian matrix

C  Continuum-continuum blocks

C
C        allocate(wrkcc(nrang2,nrang2,kccc))
	
	IREC=-1
	JDISC=19
	LENGTH=NRANG2*NRANG2

	ALLOCATE(ARRAYDA(LENGTH))
	ARRAYDA=0.0d0

	ALLOCATE(MATDA(NRANG2,NRANG2))
	LENGTH=0
	KEY=2
	
	CALL DA2(KEY,IREC,JDISC,LENGTH,NRANG2*NRANG2)
	
	LENGTH=NRANG2*NRANG2
C
        DO ICH = 1,NCHAN
          DO JCH = ICH,NCHAN
            READ (ifile) I,J
            kcc = ksym(I,J)
            IF (I.EQ.J) THEN

              READ (ifile)((matda(M,N),N=M,NRANG2),M=1,NRANG2)
	            
	      
c  Note, the diagonal block needs to be symmetrised
              do M=1,NRANG2
                do N=M,NRANG2
                  matda(N,M)=matda(M,N)
                enddo
              enddo
	    
              k=0 
              do M=1,NRANG2
               do N=1,NRANG2 
                k=k+1
	        arrayda(k)=matda(N,M)
               enddo
              enddo
	      
	      CALL DA2(2,kcc,JDISC,LENGTH,NRANG2*NRANG2)
	      
c              print*,kcc,i,j,ich,jch,(wrkcc(M,M,kcc),M=1,NRANG2)
            ELSE
              READ (ifile) ((matda(M,N),N=1,NRANG2),M=1,NRANG2)
	      
              k=0 
              do M=1,NRANG2
               do N=1,NRANG2 
                k=k+1
	        arrayda(k)=matda(N,M)
               enddo
              enddo
	      CALL DA2(2,kcc,JDISC,LENGTH,NRANG2*NRANG2) 
	      
            ENDIF
          ENDDO
        ENDDO

        CLOSE(JDISC,status='keep')

        IREC=-1
        LENGTH=0

	CALL DA3(1,IREC,JDISC,LENGTH,NRANG2*NRANG2)

	
	LENGTH=NRANG2*NRANG2
         
c  Note, transpose is written out, because input is on one side of the
c  triangle (JCH = ICH,NCHAN) and output is on other side (JCH = 1,ICH)
        DO ICH = 1,NCHAN
          DO JCH = 1,ICH
            kcc = ksym(JCH,ICH)
	    
	    CALL DA3(1,kcc,JDISC,LENGTH,NRANG2*NRANG2)
	    
              k=0 
              do M=1,NRANG2
               do N=1,NRANG2 
                k=k+1
	        matda(N,M)=arrayda(k)
               enddo
              enddo

   
             if(mod(loop,2).eq.1)then


            WRITE(jfile) ((matda(N,M),N=1,NRANG2),M=1,NRANG2)
              else
            WRITE(jfile2) ((matda(N,M),N=1,NRANG2),M=1,NRANG2)
             endif

          ENDDO
        ENDDO

       call comm_barrier()
       
       deallocate(arrayda,matda)

       CLOSE(JDISC,status='delete')


        IF (NCFGP.GT.0) THEN

C  Continuum-bound blocks


          allocate(wrkcb(NRANG2,NCFGP,NCHAN))

          DO ICH = 1,NCHAN
            READ (ifile) I,((wrkcb(M,N,I),N=1,NCFGP),M=1,NRANG2)
          ENDDO
          DO ICH = 1,NCHAN
             if(mod(loop,2).eq.1)then
            WRITE(jfile) ((wrkcb(M,N,ICH),N=1,NCFGP),M=1,NRANG2)
             else
            WRITE(jfile2) ((wrkcb(M,N,ICH),N=1,NCFGP),M=1,NRANG2)
             endif
          ENDDO

          deallocate(wrkcb)

C  Bound-bound elements

          allocate(wrkbb(NCFGP))

          DO M = 1,NCFGP
            READ (ifile) (wrkbb(N),N=M,NCFGP)
             if(mod(loop,2).eq.1)then
            WRITE(jfile) (wrkbb(N),N=M,NCFGP)
              else
            WRITE(jfile2) (wrkbb(N),N=M,NCFGP)
             endif
          ENDDO

          deallocate(wrkbb)

        ENDIF

c  Asymptotic coefficients

        if( LAMAX.GT.0 ) then
          do I = 1, NCHAN
            if(mod(loop,2).eq.1)then
            WRITE(jfile) ((2*CF(J,I,K),K=1,LAMAX),J=I,NCHAN)  !2* NRB
            else
            WRITE(jfile2) ((2*CF(J,I,K),K=1,LAMAX),J=I,NCHAN)  !2* NRB
            endif 
          enddo
        endif

        INAST2 = loop
      enddo
c     
c  Gather data for sizeBP.dat file
c
      allocate(bp1(2*nproc))
      allocate(bp2(2*nproc))
      allocate(bp3(2*nproc))
      allocate(bp4(2*nproc))
      allocate(bp5(2*nproc))
      call comm_gather(bpv1,2,bp1,2)
      call comm_gather(bpv2,2,bp2,2)
      call comm_gather(bpv3,2,bp3,2)
      call comm_gather(bpv4,2,bp4,2)
      call comm_gather(bpv5,2,bp5,2)
c
c  Write sizeBP.dat file
c
      if(iam.eq.0)then
      OPEN(UNIT=765,STATUS='UNKNOWN',FORM='FORMATTED',FILE='sizeBP.dat')
      do II=1,2*nproc,1
       NCHAN=bp1(II)
       MNP1W=bp2(II)
       MNP1=bp3(II)
       J2=bp4(II)
       JPI=bp5(II)
       IF(JPI.eq.1)JPI=0
       IF(JPI.eq.-1)JPI=1
       write(765,995)NCHAN,MNP1W,MNP1,
     A            ' 2J= ',J2,' PI ',JPI
       enddo
      endif
 995      FORMAT(3I7,A5,I7,A4,I7)
c
      deallocate(bp1)
      deallocate(bp2)
      deallocate(bp3)
      deallocate(bp4)
      deallocate(bp5)
c-----------------------------------------------------------------------
c  Dipole matrix
c
      if(irecuph.eq.0)then
      close(jfile,status='delete')
      close(jfile2,status='delete')
      endif

      if(irecupd.ne.1)goto 99

      if( IPOLPH.le.1 ) goto 99
   10 continue
c

c      PARAMETER (INML=MZNR2/MZNC2, JNML=MZNC2/MZNR2, KNML=INML+JNML)
c      PARAMETER (ND24=MZNR2*INML/KNML+MZNC2*JNML/KNML)
       inml=nrang2/ncfmax
       jnml=ncfmax/nrang2
       knml=inml+jnml
       nd24=nrang2*inml/knml+ncfmax*jnml/knml
       allocate(wrkd(nd24,nd24,2))
c
c    To be implemented...
c      if(nproc.gt.1)then
c        print *,'Using a single RECUPD.DAT dipole file....'
c        print *,'Re-run with 1 RECUPH.DAT file if necessary.'
c      endif
c
c  Open dipole matrix output file
      i1=iam/100
      i2=(iam-100*(iam/100))/10
      i3=iam-(100*(iam/100))-i2*10
      recd='RECUPD'//NUM(i1)//NUM(i2)//NUM(i3) 
      OPEN( UNIT=kfile, STATUS='UNKNOWN', FORM='UNFORMATTED',
     *      FILE=recd,access='sequential',action='write')

c Loop over pairs of symmetries for dipole

      do loop2 = 2, INAST2
       do loop1 = 1,loop2-1
        MJ    = ILSPI(1,loop1)
        MMNP2 = ILSPI(2,loop1)
        MCHAN = ILSPI(3,loop1)
        NJ    = ILSPI(1,loop2)
        MNP2  = ILSPI(2,loop2)
        NCHAN = ILSPI(3,loop2)
        mcfg  = MMNP2-MCHAN*NRANG2
        ncfg  = MNP2 -NCHAN*NRANG2
        jdif  = (abs(MJ)-abs(NJ))/2
        if( MJ*NJ.lt.0 .and. abs(jdif).le.1 ) then
          if(iprint.ge.1)write(iout,*) 'Dipole transition 2J+1:',mj,nj

c Selection rules ok.  read/write dipole matrix elements

          I = -2
          J = -2

c Continuum-continuum dipole

          do nch = 1, NCHAN
            m2 = 0
            do mch = 1, MCHAN
              if( I.eq.-2 .and. J.eq.-2 ) READ (ifile,end=98) I,J,K,L
c              print*,i,j,k,l
              IF (I.EQ.-1 .AND. J.EQ.-1) GOTO 20
              if( I.eq.nch .and. J.eq.mch ) then
                I = -2
                J = -2
                READ (ifile) ((wrkd(M,N,1),M=1,K),N=1,L)
                READ (ifile) ((wrkd(M,N,2),M=1,K),N=1,L)
                WRITE(kfile) ((wrkd(M,N,1),M=1,K),N=1,L)
                WRITE(kfile) ((wrkd(M,N,2),M=1,K),N=1,L)
              else
                WRITE(kfile) (zero,M=1,NRANG2*NRANG2)
                WRITE(kfile) (zero,M=1,NRANG2*NRANG2)
              endif
              m2 = m2 + NRANG2
            enddo
            if( m2.lt.MMNP2 ) then
              if( I.eq.-2 .and. J.eq.-2 ) READ (ifile,end=98) I,J,K,L
c              print*,i,j,k,l
              IF (I.EQ.-1 .AND. J.EQ.-1) GOTO 20
              if( I.eq.nch .and. J.eq.0 ) then
                if( mcfg.ne.L ) then
              write(iout,*) 'STOP: mcfg and L must be the same:',mcfg,L
                  stop
                endif
                I = -2
                J = -2
                READ (ifile) ((wrkd(M,N,1),M=1,K),N=1,L)
                READ (ifile) ((wrkd(M,N,2),M=1,K),N=1,L)
                WRITE(kfile) ((wrkd(M,N,1),M=1,K),N=1,L)
                WRITE(kfile) ((wrkd(M,N,2),M=1,K),N=1,L)
              else
                WRITE(kfile) (zero,M=1,NRANG2*mcfg)
                WRITE(kfile) (zero,M=1,NRANG2*mcfg)
              endif
            endif
            WRITE(kfile) ((ABUTL,M=1,NRANG2),N=1,MCHAN)
            WRITE(kfile) ((ABUTV,M=1,NRANG2),N=1,MCHAN)
          enddo
 
c Bound-continuum dipole

          if( ncfg.gt.0 ) then
            do mch = 1, MCHAN
              if( I.eq.-2 .and. J.eq.-2 ) READ (ifile,end=98) I,J,K,L
c              print*,i,j,k,l
              IF (I.EQ.-1 .AND. J.EQ.-1) GOTO 20
              if( I.eq.0 .and. J.eq.mch ) then
                if( ncfg.ne.K ) then
                  print *,'STOP: ncfg and K must be the same:',ncfg,K
                  stop
                endif
                I = -2
                J = -2
                READ (ifile) ((wrkd(M,N,1),M=1,K),N=1,L)
                READ (ifile) ((wrkd(M,N,2),M=1,K),N=1,L)
                WRITE(kfile) ((wrkd(M,N,1),M=1,K),N=1,L)
                WRITE(kfile) ((wrkd(M,N,2),M=1,K),N=1,L)
              else
                WRITE(kfile) (zero,M=1,ncfg*NRANG2)
                WRITE(kfile) (zero,M=1,ncfg*NRANG2)
              endif
            enddo

c Bound-bound dipole

            if( mcfg.gt.0 ) then
              if( I.eq.-2 .and. J.eq.-2 ) READ (ifile,end=98) I,J,K,L
c              print*,i,j,k,l
              IF (I.EQ.-1 .AND. J.EQ.-1) GOTO 20
              if( I.eq.0 .and. J.eq.0 ) then
                if( mcfg.ne.L .or. ncfg.ne.K ) then
                  print *,'STOP: mcfg,ncfg and L,K=',mcfg,L,ncfg,K
                  stop
                endif
                I = -2
                J = -2
                READ (ifile) ((wrkd(M,N,1),M=1,K),N=1,L)
                READ (ifile) ((wrkd(M,N,2),M=1,K),N=1,L)
              else
                do N=1,mcfg
                  do M=1,ncfg
                    wrkd(M,N,1)=zero
                    wrkd(M,N,2)=zero
                  enddo
                enddo
              endif
              NDIMEN = NRANG2
              NTIMES = (ncfg-1)/NDIMEN + 1
              n2 = 0
              do nt = 1,NTIMES
                n22 = min(n2+NDIMEN,ncfg)
                WRITE(kfile) ((wrkd(M,N,1),M=n2+1,n22),N=1,mcfg)
                WRITE(kfile) ((wrkd(M,N,2),M=n2+1,n22),N=1,mcfg)
                n2 = n22
              enddo
            endif
          endif

c Write zeros for dipole matrix Buttle correction

   20     READ (ifile,iostat=ioio) I,J,K,L
          if(ioio.ne.0)goto 98
c          print*,i,j,k,l
          if( I.ne.-1 .and. J.ne.-1 .and. K.ne.-1 .and. L.ne.-1 ) then
            print*,'WARNING: dipole matrices not finished off:',i,j,k,l
          endif
          n2 = NCHAN*NRANG2
          if( n2.LT.MNP2 ) then
            WRITE(kfile) ((ABUTL,J=1,MNP2-n2),I=1,MCHAN)
            WRITE(kfile) ((ABUTV,J=1,MNP2-n2),I=1,MCHAN)
          endif
          do MCH = 1,MCHAN
            WRITE(kfile) ((BBUTL,J=1,NCHAN),I=1,NRANG2)
            WRITE(kfile) ((BBUTV,J=1,NCHAN),I=1,NRANG2)
          enddo
          m2 = MCHAN*NRANG2
          if( m2.LT.MMNP2 ) then
            WRITE(kfile) ((BBUTL,J=1,NCHAN),I=1,MMNP2-m2)
            WRITE(kfile) ((BBUTV,J=1,NCHAN),I=1,MMNP2-m2)
          endif
          WRITE(kfile) ((BBUTL,J=1,NCHAN),I=1,MCHAN)
          WRITE(kfile) ((BBUTV,J=1,NCHAN),I=1,MCHAN)

c CLEBSCH GORDAN COEFFICIENTS FOR THE POLARIZABILITY CALC

          READ (ifile) MAXM1
          if(MAXM1.gt.MZLR2)then
            print *,'STOP! Increase dimension MZLR2 to:',MAXM1
            stop
          endif

          READ (ifile) (temp(M),M=1,MAXM1)
          WRITE(kfile) MAXM1,(temp(M),M=1,MAXM1)

c ANGULAR INTEGRALS NEEDED FOR OUTER REGION INTEGRATION

          if(outer) then
            READ (ifile,end=96) ((AC(I,J),J=1,MCHAN),I=1,NCHAN)
            READ (ifile) ((BLC(I,J),J=1,MCHAN),I=1,NCHAN)
            READ (ifile) ((BVC(I,J),J=1,MCHAN),I=1,NCHAN)
        if(iprint.ge.1)write(iout,*) 'outer region contributions found'
            go to 97
          endif
   96 write(iout,*)'outer region contributions not found:zeros inserted'
          do j=1,MZCHF
            do i=1,MZCHF
              AC(i,j)  = zero
              BLC(i,j) = zero
              BVC(i,j) = zero
            enddo
          enddo
   97     WRITE(kfile) ((AC (I,J),J=1,MCHAN),I=1,NCHAN)
          WRITE(kfile) ((BLC(I,J),J=1,MCHAN),I=1,NCHAN)
          WRITE(kfile) ((BVC(I,J),J=1,MCHAN),I=1,NCHAN)
        endif
       enddo
      enddo

   98 if(iprint.ge.1)write(iout,*) 'RECUPDXXX created'
      close(kfile)
      deallocate(wrkd)
   99 continue 

      if(irecuph.eq.1)then
      close(jfile)
      close(jfile2)
      endif

      if(idelete.ne.0)then
       close(ifile,status='delete')
      else
       close(ifile)
      endif

C      stop 'Normal End'
      end


      SUBROUTINE DA2(KEY,IREC,JDISC,LENGTH,LREC)
      use comm_interface, only : iam
      use pdam, only : arrayda
      IMPLICIT REAL*8 (A-H,O-Z)
C-----------------------------------------------------------------------
C
C    TO STORE A LARGE ARRAY IN A DA FILE OF RECORD LENGTH 8*LREC BYTES.
C
C    KEY = 1 FOR READ,
C        = 2 FOR WRITE,
C        = 0 FOR FINDING NUMBER OF DA RECORDS A GIVEN ARRAY TAKES.
C
C    IREC= (ON CALL) POINTER TO FIRST DA RECORD FOR ARRAY,
C        = 0 FOR OPENING DA FILE (BY NAME),
C        =-1 FOR OPENING DA FILE (SCRATCH),
C        = (ON RETURN) POINTER TO NEXT AVAILABLE DA RECORD.
C
C    JDISC = DA FILE UNIT NUMBER.
C
C    ARRAY(LENGTH) = ARRAY TO READ OR WRITE.
C
C-----------------------------------------------------------------------
C      PARAMETER (LREC=3600)
C
C      DIMENSION ARRAY(LENGTH)

c       **** parallel ****
        common /pstg1block/nprocstg1
        character*1 NUM(0:9)	
        DATA NUM /'0','1','2','3','4','5','6','7','8','9'/
c	**** parallel ****
C-----------------------------------------------------------------------
C

      IF (IREC.GT.0) GOTO 20
C
      IRECL=8*LREC
C
      IF (IREC.LT.0) THEN

      i1=iam/100
      i2=(iam-100*(iam/100))/10
      i3=iam-(100*(iam/100))-i2*10


        OPEN (JDISC,FILE='SCRATCHER'//NUM(i1)//NUM(i2)//NUM(i3),
     X       ACCESS='DIRECT',FORM='UNFORMATTED',
     X       RECL=IRECL,action='write')
        GOTO 10
      ENDIF
C
C
   10 CONTINUE
      IREC = 1
C
   20 CONTINUE
      IF (LENGTH.EQ.0) RETURN
      I2 = 0
   30 CONTINUE
      I1 = 1
      I2 = MIN(LREC,LENGTH)
      IF (KEY.EQ.0) GOTO 40
C
        WRITE (JDISC,REC=IREC) ARRAYDA(I1:I2)
C
   40 CONTINUE
c      IREC = IREC + 1
c      IF (I2.LT.LENGTH) GOTO 30
C
      END
      SUBROUTINE DA3(KEY,IREC,JDISC,LENGTH,LREC)
      use comm_interface, only : iam
      use pdam, only : arrayda
      IMPLICIT REAL*8 (A-H,O-Z)
C-----------------------------------------------------------------------
C
C    TO STORE A LARGE ARRAY IN A DA FILE OF RECORD LENGTH 8*LREC BYTES.
C
C    KEY = 1 FOR READ,
C        = 2 FOR WRITE,
C        = 0 FOR FINDING NUMBER OF DA RECORDS A GIVEN ARRAY TAKES.
C
C    IREC= (ON CALL) POINTER TO FIRST DA RECORD FOR ARRAY,
C        = 0 FOR OPENING DA FILE (BY NAME),
C        =-1 FOR OPENING DA FILE (SCRATCH),
C        = (ON RETURN) POINTER TO NEXT AVAILABLE DA RECORD.
C
C    JDISC = DA FILE UNIT NUMBER.
C
C    ARRAY(LENGTH) = ARRAY TO READ OR WRITE.
C
C-----------------------------------------------------------------------
C      PARAMETER (LREC=3600)
C
C      DIMENSION ARRAY(LENGTH)

c       **** parallel ****
        common /pstg1block/nprocstg1
        character*1 NUM(0:9)	
        DATA NUM /'0','1','2','3','4','5','6','7','8','9'/
c	**** parallel ****
C-----------------------------------------------------------------------
	
      IF (IREC.GT.0) GOTO 20
C
      IRECL = 8*LREC
C
      IF (IREC.LT.0) THEN
      i1=iam/100
      i2=(iam-100*(iam/100))/10
      i3=iam-(100*(iam/100))-i2*10

      write(0,*)'read','SCRATCHER'//NUM(i1)//NUM(i2)//NUM(i3)

        OPEN (JDISC,FILE='SCRATCHER'//NUM(i1)//NUM(i2)//NUM(i3),
     X       ACCESS='DIRECT',FORM='UNFORMATTED',
     X       RECL=IRECL,ACTION='read',status='old')
        GOTO 10
      ENDIF
C
C
   10 CONTINUE
      IREC = 1
C
   20 CONTINUE
      IF (LENGTH.EQ.0) RETURN
      I2 = 0
   30 CONTINUE
      I1 = 1
      I2 = MIN(LREC,LENGTH)
      IF (KEY.EQ.0) GOTO 40
C
        READ (JDISC,REC=IREC) ARRAYDA(I1:I2)
C
   40 CONTINUE
c      IREC = IREC + 1
c      IF (I2.LT.LENGTH) GOTO 30
C
      END
