!
!                                                           3/10/2019 
!      parallel dipole matrix formation : pstgd_split.f
!
!      CPB 2011   : ndipoles * nproc_per_subworld = total nos of processors
!
!      INPUT DECK : dstgd
!
!        nproc_per_subworld=4     !procs per dipole
!        ndip=1                   ! 1 length, 0 velocity
!        ndipole=3                !number of dipoles
!
!      requires : sizeH.dat/sizeBP.dat                              !produced automatically by pstg2r/precupd
!               : H.DAT                                             !produced automatically by pstg3rd
!               : STG2D.DAT/STG2D.XX  or RECUPD.DAT/RECUPD.XX files !produced automatically by pstg2r/precupd
!               : IORDERXX                                          !produced automatically by pstg3rd
!               : DIPORDER                                          !produced automatically by pstg3rd
!               : symmetryXXX                                       !produced automatically by pstg3rd
!           
!               : PARAM                                         ! user supplied
!
!
!     Conditions  :    NPROC <  MIN( Hamiltonian dimension for either sym )
!
!     Recommended number of processors   =  MAX (Hamiltonian dimension) / MAX (number of channels)
!
!      ie 17314 matrix with 701 channels   ~ 24/25 processors 
!
!     Eigenvectors should be minimally allocated for both symmetries
!
!     Default run produces only Dipole matrices in length guage to save 
!     further memory, controlled by NDIP flag in DMAT2 
!
!      mar 2015    .... put back IORDER READS from IORDERXX files
!
!      oct 2019    .... cleaned up inefficient do loops  + stream option for DSCRATCH files
!
!****************************************************************************
      module comm_interface

      implicit none
      include "mpif.h"
      public comm_init          ! Initialize MPI
      public comm_barrier       ! MPI barrier
      public comm_barriersub
      public comm_finalize      ! Terminate MPI
      public comm_split         ! Split worlds
      integer, public  :: icolour
      integer, public  :: masteriam
      integer, public  :: masternproc
      integer, public  :: iam,nproc
      integer, public  :: new_nodes
      integer, public  :: new_comm 
      integer, public  :: ndipole,nproc_per_subworld,ndip
      integer, public  :: nbp=0,nls=0,kslp(250,6)=0,mxmat,mxxchan !changing 250 needs changing below mpi_bcast
      SAVE
      private
      integer :: mpicom

      CONTAINS

!-----------------------------------------------------------------------------
      subroutine comm_init()
      implicit none
      integer :: ier

      mpicom = MPI_COMM_WORLD
      call mpi_init(ier)
      call mpi_comm_rank(mpicom, masteriam, ier)
      call mpi_comm_size(mpicom, masternproc, ier)

      return
      end subroutine comm_init
!-----------------------------------------------------------------------------
      subroutine comm_barrier()
      implicit none
      integer :: ier

      call mpi_barrier(mpicom, ier)

      return
      end subroutine comm_barrier
!-----------------------------------------------------------------------------
      subroutine comm_barriersub()
      implicit none
      integer :: ier

      call mpi_barrier(new_comm, ier)

      return
      end subroutine comm_barriersub
!-----------------------------------------------------------------------------
       subroutine comm_finalize()
      implicit none
      integer :: ier

      call mpi_finalize(ier)

      return
       end subroutine comm_finalize
!-----------------------------------------------------------------------------
       subroutine comm_split(icolour)
       implicit none
       integer :: ier
       integer :: icolour
       call mpi_comm_split(mpicom,icolour,masteriam,new_comm,ier)
       call mpi_comm_rank(new_comm,iam,ier)
       call mpi_comm_size(new_comm,nproc,ier)
       return
       end subroutine comm_split
!----------------------------------------------------------------------------
      end module comm_interface
!*****************************************************************************
      module param1
      implicit real*8(a-h,o-z)
      INCLUDE 'PARAM'
      end module param1
!***********************************************************************
!***********************************************************************
      module ciinput
      implicit real*8(a-h,o-z)
      integer :: LAMAX,LRANG2,LRGL2,MNP2,NAST,NCHAN,NELC,NPTY2,NSPN2,
     X           NZ,MORE2,KFLAG
      integer,allocatable :: ISAT(:),LAT(:),L2P(:),NCONAT(:),KJ(:)
      real*8 :: BSTO,RA
      real*8,allocatable :: CF(:,:,:),COEFF(:,:),ENAT(:),
     X                    WMAT(:,:),VALUE(:)
      end module ciinput
!***********************************************************************
!***********************************************************************
      module dbut1
      implicit real*8(a-h,o-z)
      integer,allocatable :: NBUTD(:),K2P(:)
      real*8,allocatable :: EBUTD(:,:),CBUTD(:,:)
      end module dbut1
!***********************************************************************
!***********************************************************************
      module eigvec
      implicit real*8(a-h,o-z)
      real*8,allocatable :: VI(:,:),VJ(:,:)
      end module eigvec
!***********************************************************************
!***********************************************************************
      module dipoleorder
      implicit real*8(a-h,o-z)
      integer,dimension(100,2) :: iorderdip
      integer,dimension(7,100) :: idipsym 
      integer :: nosofdipolepairs,ISERIAL,nsym
      end module dipoleorder
!***********************************************************************
!*********************************************************************** 
      module symrec
      implicit real*8(a-h,o-z)
      CHARACTER*10 :: SYM1,SYM2
      end module symrec
!***********************************************************************
!***********************************************************************
      module matrix
      real*8,allocatable :: abutl(:,:),abutv(:,:),bbutl(:,:),bbutv(:,:)
      real*8,allocatable :: dum(:)
      end module matrix   
!***********************************************************************
!***********************************************************************   
      module part
      real*8,allocatable :: eigens(:,:),ENDS(:,:),SI(:)
      real*8 :: TRACE
      integer,allocatable :: NRANG1(:)
      integer :: NRANG2,IPRCENT
  
!       COMMON/PART/EIGENS(MZNR2,MZLR2),ENDS(MZNR2,MZLR2),SI(MZCHF),
!     X            TRACE,NRANG1(MZLR2),NRANG2,IPRCENT
      end module part

        program pstgdsplit
        use comm_interface
        implicit real*8(a-h,o-z)
        character*5 :: cdum,a5
        character*4 :: a4 
        logical ex,exiorder
        include 'mpif.h'	
        NAMELIST/PREDIP/nproc_per_subworld,ndip,ndipole
!
        ITAPE4=14     ! CPB 2004
!
        nproc_per_subworld=4     !procs per dipole
        ndip=1                   ! 1 length, 0 velocity
        ndipole=3                !number of dipoles
!
       INQUIRE (FILE='dstgd',EXIST=EX)
!
        call comm_init()
!
         if(.NOT.EX)then
          continue
         else
          open(110,file='dstgd',status='unknown',form='formatted')
          read(110,predip)
          close(110)
C
C          All dipoles the same number of procs
C
           if(nproc_per_subworld.gt.0)then
            icolour=masteriam/nproc_per_subworld
            ntot=ndipole*nproc_per_subworld
C
c           write(0,*)ntot,masternproc
           if(ntot.ne.masternproc)then
           write(0,*)'number of processors incorrect .... sighhh'
           stop
           endif
C	    
           endif
C
C
!
!
           if(nproc_per_subworld.lt.0)then
      open(111,file='DISTRIBUTED.DAT',status='unknown',form='formatted')
           DO I=0,ndipole-1
           READ(111,*)istart,iend
C           write(0,*)'istart',istart,'iend',iend
           if((masteriam.ge.istart).and.(masteriam.le.iend))then
           icolour=I
C           write(0,*)istart,iend,masteriam
           nproc_per_subworld=iend-istart+1
           endif
           ENDDO
           close(111)
          endif
         endif
!
!
        if(masteriam.eq.0)then
C
        INQUIRE(FILE='sizeBP.dat',EXIST=EX)
C
        IF(EX)then                            ! IF
        OPEN (ITAPE4,FILE='D00',STATUS='UNKNOWN',FORM='UNFORMATTED')
        OPEN (UNIT=39,FILE='sizeBP.dat',STATUS='OLD',FORM='FORMATTED')
        write(6,*)'writing D00 from sizeBP.dat'
C
C       consider the following dummy variables, not actual ones
C
C   Comment yes the BP and DARC should have the same form ... but 
C   it is taken care of here also.
C
         WRITE(ITAPE4)ndipole
        J=0
        do I=1,ndipole             !DARC
          READ(39,995,end=40)IDUMA,IDUM2,IDUM3,A5,ISPN,A4,ILRGL
          write(6,995)IDUMA,IDUM2,IDUM3,A5,ISPN,A4,ILRGL

          J=J+1
          kslp(J,1)=IDUMA
          kslp(J,2)=IDUM2
          kslp(J,3)=IDUM3
          kslp(J,4)=0           !weird but spin,l and parity
          kslp(J,5)=ISPN
          kslp(J,6)=ILRGL

          READ(39,995,end=40)IDUMB,IDUM2,IDUM3,A5,JSPN,A4,JLRGL

          J=J+1
          kslp(J,1)=IDUMB
          kslp(J,2)=IDUM2
          kslp(J,3)=IDUM3
          kslp(J,4)=0
          kslp(J,5)=JSPN
          kslp(J,6)=JLRGL

c          write(6,995)IDUMB,IDUM2,IDUM3,A5,JSPN,A4,JLRGL
c          if(ilrgl.eq.-1)then  !darc signature
c          jlrgl=0
c          ilrgl=1
c          endif
c          if(jlrgl.eq.-1)then !darc signature
c          ilrgl=0
c          jlrgl=1
c          endif
          IZERO=0
          JZERO=0
          WRITE (ITAPE4) IZERO,ISPN,ILRGL,JZERO,JSPN,JLRGL
        enddo
         close(39) 
         goto 41
  40     continue
         close(39)
         OPEN (UNIT=39,FILE='sizeBP.dat',STATUS='OLD',FORM='FORMATTED')
         REWIND(ITAPE4)
         WRITE(ITAPE4)ndipole
         J=0
        do I=1,ndipole              !BP
          READ(39,995)IDUM1,IDUM2,IDUM3,A5,ISPN,A4,ILRGL

          READ(39,995)IDUM1,IDUM2,IDUM3,A5,JSPN,A4,JLRGL

          IZERO=0
          JZERO=0
          WRITE (ITAPE4) IZERO,ISPN,ILRGL,JZERO,JSPN,JLRGL
        enddo
          write(0,*)'Old pstgjk .... Upgrade your pstgjk_dip '
          close(39)
          goto 41
      ELSE                                   ! IF
        write(0,*)'writing D00 from sizeH.dat',ndipole
        OPEN (UNIT=39,FILE='sizeH.dat',STATUS='OLD',FORM='FORMATTED')
        OPEN (ITAPE4,FILE='D00',STATUS='UNKNOWN',FORM='UNFORMATTED')
      	WRITE(ITAPE4)ndipole
         J=0
        do I=1,ndipole               !LS
      read(39,3010)charb,IDUMA,charc,IDUM2,chara,IDUM3,ISPN,ILRGL,IPTY
          J=J+1
          kslp(J,1)=IDUMA
          kslp(J,2)=IDUM2
          kslp(J,3)=IDUM3
          kslp(J,4)=ISPN
          kslp(J,5)=ILRGL
          kslp(J,6)=IPTY

      read(39,3010)charb,IDUMB,charc,IDUM2,chara,IDUM3,JSPN,JLRGL,JPTY

          J=J+1
          kslp(J,1)=IDUMB
          kslp(J,2)=IDUM2
          kslp(J,3)=IDUM3
          kslp(J,4)=JSPN           
          kslp(J,5)=JLRGL
          kslp(J,6)=JPTY

         WRITE (ITAPE4) ISPN,ILRGL,IPTY,JSPN,JLRGL,JPTY
        enddo
 995  FORMAT(3I7,A5,I7,A4,I7)
3010  FORMAT (a8,I5,a9,I7,a7,I7,3I7)
      ENDIF                                  ! IF  
C
        close(39)
        endif
C 

 41     continue
 
        call MPI_Bcast(kslp,1500,mpi_integer,0,mpi_comm_world,ier)

        call comm_barrier()
!
        if(iam.eq.0)then
        call cpu_time(timei)
        endif

        call comm_split(icolour)	
!
        I=2*icolour+1

        mxmat=max(kslp(I,3),kslp(I+1,3))
        mxxchan=max(kslp(I,1),kslp(I+1,1))
        if(iam.eq.0)write(6,*)'mxmat =', mxmat,' mxchan=',mxxchan 
!
        izero=-1
        if(iam.eq.0)izero=icolour
        call MPI_Bcast(izero,1,mpi_integer,0,new_comm,ier)
!        print*,'masteriam= ',masteriam,'iam',iam,icolour
!        goto 100	
!	
       call pstgd()
!
       call comm_barrier()
!
!     100   continue
!
       call mpi_comm_free(new_comm,ier)
!
!
      if(iam.eq.0)then
       call cpu_time(timef)
      time=timef-timei
C      write(0,999) time/60.,nproc
999   format(//1x,'CPU TIME=',f9.3,' MIN  -- processors=:',i4)
C
      endif
C
 1000 continue
      call comm_barrier() 
      call comm_finalize()
C
C      
      end 

        subroutine pstgd
        use comm_interface 	
        use param1
        use eigvec
	use dipoleorder
        use symrec
        use part, only : nrang2
C
C
C
        implicit real*8(a-h,o-z)
        logical exh,ex,exs,ex1,ex2,exiorder,exdipole,exls,exbp
        CHARACTER :: NUM(0:9)*1
        CHARACTER*10 :: BPDIP
        CHARACTER*8 :: IORD,charb
        CHARACTER*9 :: LSDIP,charc
        CHARACTER*7 :: chara
        CHARACTER*5 :: CDUM
        DATA NUM/'0','1','2','3','4','5','6','7','8','9'/
C
C
C
C
       COMMON /INFORM/IREAD,IWRITE,IPUNCH
       COMMON /MEMORY/MEM1,MREC1
c       COMMON/PART/EIGENS(MZNR2,MZLR2),ENDS(MZNR2,MZLR2),SI(MZCHF),
c     X            TRACE,NRANG1(MZLR2),NRANG2,IPRCENT
       COMMON /RECORD/ILSPI(4,MZSLP),IORDER(MZCHF,MZSLP)
       COMMON /DISTAP/IDISC1,IDISC2,IDISC3,IDISC4,ITAPE1,ITAPE2,ITAPE3,
     A       ITAPE4,ITAPE5
C
C
        ITAPE1=9      ! CPB 2004
        ITAPE4=14     ! CPB 2004
        ITAPE5=16     ! CPB 2005
C
        nsym=1
        ISERIAL=0
        nofdip=0
C
C
C       a prototype code to include dipole matrices 
C       within the parallel suite of codes.
C
C        UNIT NUMBERS  (10 ) : H.DAT
C                      (9)   : STG2DXXX/RECUPDXXX
C                      (8)   : eigenvectors of symmetry 1 (odd)   
C                      (12)  : eigenvectors of symmetry 2 (even)
C                      (13)  : DIPORDER
C                   
C
C
        i1=icolour/10
        i2=icolour-(10*i1)
C       
        OPEN(88,file='routd'//NUM(i1)//NUM(i2),status='unknown',
     X       form='formatted')    
C
C
        if(masteriam.eq.0)then
        write(6,*)
        write(6,*)
        call cpu_time(timei)
        write(6,*)'****************************************************'
        write(6,*)'                                                    '
        write(6,*)'                                PSTGD               '
        write(6,*)'                                                    '
        write(6,*)'       nproc_per_subworld = ', nproc_per_subworld
        write(6,*)'     length 1 , velocity 0 =', ndip
        write(6,*)'         Nos of   dipoles  =', ndipole
        write(6,*)'****************************************************'
        endif
C
        ITAPE1=9
C
C       Set counter for the number of symmetries
C
C
C
C       Parallel STG2D/RECUPD
C
C
C
        INQUIRE (FILE='RECUPD000',EXIST=EX)
        IF (.NOT.EX) THEN
          INQUIRE (FILE='STG2D000',EXIST=EX)
          IF (.NOT.EX) THEN
             WRITE (6,*)
             WRITE (6,*) 'Neither RECUPD000 nor STG2D000 exist'
             WRITE (6,*)
             WRITE (6,*)' ... looking for serial passing files ...'
             WRITE (6,*) 
          ELSE
	  
	i0=icolour/100
        i1=icolour/10
        i2=icolour-i1*10
		  
            OPEN (UNIT=ITAPE1,FILE='STG2D'//NUM(i0)//NUM(i1)//NUM(i2),
     Z            STATUS='OLD',
     A            FORM='UNFORMATTED')
      if(iam.eq.0)write(88,*)'OPEN STG2D'//NUM(i0)//NUM(i1)//NUM(i2),
     Z                        ' in subworld ',icolour
                       ISERIAL=1
          ENDIF

C
        ELSE
	
	i0=icolour/100
        i1=icolour/10
        i2=icolour-i1*10
	
          OPEN (UNIT=ITAPE1,FILE='RECUPD'//NUM(i0)//NUM(i1)//NUM(i2),
     Z          STATUS='OLD',
     A          FORM='UNFORMATTED')
                       ISERIAL=1
      if(iam.eq.0)write(88,*)'OPEN RECUPD'//NUM(i0)//NUM(i1)//NUM(i2),
     Z                        ' in subworld ',icolour
        ENDIF
C
C
         nosofdipolepairs=0
C
         III =0      ! tmp counter for next section
C
        INQUIRE (FILE='DIPORDER',EXIST=EXDIPOLE)
        IF (.NOT.EXDIPOLE) THEN
       STOP 'REQUIRE an DIPORDER file linking appropriate even-odd syms'
        ELSE
        OPEN (UNIT=13,FILE='DIPORDER',STATUS='OLD',FORM='FORMATTED')
	   DO II=1,100
	   READ(13,*,end=5)iorderdip(II,1),iorderdip(II,2)
	   III=III+1
           ENDDO	   
        ENDIF
C
 5      continue
C   
        if(MOD(III,2).ne.0)stop'need even number of symmetries'
C
        nosofdipolepairs=III/2
C
        if(iam.eq.0)then
        write(88,*) 
        write(88,*)'Total number of dipole pairs =',nosofdipolepairs	
        write(88,*)
        endif
C
C
C
  10    CONTINUE
C
C
        INQUIRE (FILE='H.DAT',EXIST=EXH)
        IF (.NOT.EXH) THEN
       STOP 'REQUIRE an H.DAT with a couple of symmetries to be present'
        ELSE
        OPEN (UNIT=10,FILE='H.DAT',STATUS='OLD',FORM='UNFORMATTED')
        ENDIF

C       get the dimensions from the H.DAT file
C
C         call comm_barrier()
C         call comm_finalize()
C         stop
C
C
C
        iopt1=0
C	         
        if(nsym.eq.1)call read1     ! read N electron information of H.DAT file
C
C
C      
        nsym=nsym-1+(icolour*2)     !  just to get the indexing right, for the first symmetry
        if(iam.eq.0)write(88,*)nsym
C
C
C         ******************************************************************
C
C         MAIN LOOP  begins here
C
C         ******************************************************************
C	
        DO I=1,99
        i1=i/10
        i2=i-(10*i1)

        OPEN(8,file='IORDER'//NUM(i1)//NUM(i2),status='old',
     X       form='formatted',iostat=ierr)

        if(ierr.ne.0)then

        close(8)

        DO J=1,MZCHF
          IORDER(J,I)=J
         ENDDO

        else

        if(masteriam.eq.0)write(6,*)'IORDER'//NUM(i1)//NUM(i2),' found'

        DO J=1,MZCHF 
          READ(8,*,end=19)IORDER(J,I)
        ENDDO
 
 19     continue
        close(8)
 
        endif
      
        ENDDO

C
  20    CONTINUE
C
        nsym=nsym+1
        nofdip=1
C
        if(nsym.gt.(2*nosofdipolepairs))then
        write(0,*)'should not get here'
        goto 997     !  finish
        endif
C
C
C    ************ H.DAT SHUFFLE ************************
C
        if(icolour.gt.0)then
        kkk=0 
        do jjj=1,icolour
           kkk=kkk+1
           nmap=iorderdip(kkk,2)
              if(nmap.eq.kkk)then
c              if(iam.eq.0)write(0,*)'calling read2 dum',nmap,icolour
              call read2(iopt1,nmap,1)
              endif
           kkk=kkk+1
           nmap=iorderdip(kkk,2)
              if(nmap.eq.kkk)then 
c              if(iam.eq.0)write(0,*)'calling read2 dum',nmap,icolour
              call read2(iopt1,nmap,1)
              endif
        enddo
C
        endif
C
C    ****************************************************
C
C
C
        nmap=iorderdip(nsym,2)
        i1=nmap/10
        i2=nmap-(10*(nmap/10))
        sym1='symmetry'//NUM(i1)//NUM(i2)
C
C
        if(iam.eq.0)write(88,*)'nmap =',nmap
C
        if(nsym.eq.nmap)then
c        if(iam.eq.0)write(6,*)'calling read2 dum',nmap,icolour
         call read2(iopt1,nmap,1)
        endif
        
C
        itmp2=nsym+1
C
        nmap=iorderdip(itmp2,2)
C
c        if(iam.eq.0)write(88,*)'nmap =',nmap
         
        if(itmp2.eq.nmap)then
c        if(iam.eq.0)write(6,*)'calling read2 dum',nmap,icolour
         call read2(iopt1,nmap,1)
        endif 
C	
C
C
C
C
        INQUIRE (FILE=sym1,EXIST=EX1)
        IF (.NOT.EX1) THEN
        write(88,*)'REQUIRE an symmetryXX odd file to be present',I1,I2
         call comm_barrier()
         call comm_finalize()
         stop
        ENDIF
C
C
C
        nsym=nsym+1
        nmap=iorderdip(nsym,2)
        i1=nmap/10
        i2=nmap-(10*(nmap/10))
        sym2='symmetry'//NUM(i1)//NUM(i2)
C
        INQUIRE (FILE=sym2,EXIST=EX2)
        IF (.NOT.EX2) THEN
        write(0,*)' REQUIRE an symmetryXX even file to be present',I1,I2
         call comm_barrier()
         call comm_finalize()
         stop
        ENDIF
C
C
C
  230   CONTINUE
C
c
         call comm_barrier()
         call dmat
         call comm_barrier()
C
        if(iam.eq.0)then
        i1=icolour/10
        i2=icolour-(10*(icolour/10))
        OPEN(18,file='DSCRATCH_'//NUM(i1)//NUM(i2),status='unknown',
     X       form='unformatted')
        OPEN(19,file='DSCRATCH2_'//NUM(i1)//NUM(i2),status='unknown',
     X       form='unformatted')
        close(18,status='delete')
        close(19,status='delete')
        endif
        
C
C
C
  997    continue
      
C
C
          return	
C
C        close(10)
        end
      SUBROUTINE DMAT
      use comm_interface
      use param1
      use ciinput
      use dipoleorder
      use matrix
      use part, only : nrang2
      IMPLICIT REAL*8 (A-H,O-Z)
C
C
C
C-----------------------------------------------------------------------
C
C     WRITES 'D FILE' FOR INTERMEDIATE COUPLING ONTO TAPE4,
C
C-----------------------------------------------------------------------
C
C
c      PARAMETER (MXMAT=MZCHF*MZNR2+MZNC2)
c      PARAMETER (MXTRI=(MXMAT*(MXMAT+1))/2)
c      PARAMETER (MXD1=MZNC2/MZNR2,MXD2=MZNR2/MZNC2,MXD3=MXD1+MXD2,
c     A          MXDM=MZNC2*MXD1/MXD3+MZNR2*MXD2/MXD3)
c      PARAMETER (MXDMAT=4*MXMAT*MZCHF+2*MXMAT*MXDM+2*MZNR2*MXDM,
c     A          MXRSCT=MXTRI+MZCHF*MXMAT+MZNR2*MXMAT)
c      PARAMETER (MXM1=MXDMAT/MXRSCT,MXM2=MXRSCT/MXDMAT,MXM3=MXM1+MXM2,
c     A          MXMATX=MXDMAT*MXM1/MXM3+MXRSCT*MXM2/MXM3+1)
c      PARAMETER (MXDUM=MXMATX-4*MXMAT*MZCHF)
C
      CHARACTER FILNAM*7,NUM(0:9)*1
C
      DIMENSION IMEM(2,99)
C      DIMENSION CGC(MZLR2),AC(MZCHF,MZCHF),
C     A  BLC(MZCHF,MZCHF),BVC(MZCHF,MZCHF)
C
      REAL*8,ALLOCATABLE :: CGC(:),AC(:,:),BLC(:,:),BVC(:,:)
C
C      COMMON /BASIC1/BSTO,RA,NELC,NZ,LRANG2,NRANG2,MAXNHF(MZLR2),
C     A       MAXNLG(MZLR2),MAXNC(MZLR2),LRANG1
      COMMON /CASES/MORE,MSKIP,IPWINIT,IPWFINAL,IPOLPH,INAST
      COMMON /DISTAP/IDISC1,IDISC2,IDISC3,IDISC4,ITAPE1,ITAPE2,ITAPE3,
     A       ITAPE4,ITAPE5
      COMMON /INFORM/IREAD,IWRITE,IPUNCH
C      COMMON /MATRIX/ABUTL(MXMAT,MZCHF),ABUTV(MXMAT,MZCHF),
C     A       BBUTL(MXMAT,MZCHF),BBUTV(MXMAT,MZCHF),DUM(MXDUM)
      COMMON /MEMORY/MEM1,MREC1
      COMMON /NBUG/NBUG1,NBUG2,NBUG3,NBUG4,NBUG5,NBUG6,NBUG7,NBUG8,NBUG9
      COMMON /RCASES/NOT1,NOT2,NOT3,NPROG,NCFGP
      COMMON /RECORD/ILSPI(4,MZSLP),IORDER(MZCHF,MZSLP)
      COMMON /NRBDIP/MAXLD
c      COMMON /PART/EIGENS(MZNR2,MZLR2),ENDS(MZNR2,MZLR2),SI(MZCHF),
c     X            TRACE,NRANG1(MZLR2),NRANG2,IPRCENT
C
C
      DATA NUM/'0','1','2','3','4','5','6','7','8','9'/
      SAVE IPAIRCOUNT
C-----------------------------------------------------------------------
C
C     INITIALIZE
C
C
      IPAIRCOUNT=1+icolour             !!!!! change 2011  !!!!!
C
C
C
c      IF(IPAIRCOUNT.eq.1)THEN   
      if(ndip.eq.1)then
       allocate(abutl(mxmat,mxxchan),stat=ierr)    ! buttl gets allocated in dmat2
       if(ierr.ne.0)stop'failure allocating abutl'
      else
       allocate(abutv(mxmat,mxxchan),stat=ierr)    ! buttv gets allocated in dmat2
       if(ierr.ne.0)stop'failure allocating abutv'
      endif
c      ELSE
c       if(ndip.eq.1)then
c        deallocate(abutl,stat=ierr)
c        if(ierr.ne.0)stop'failure to deallocate abutl'
c        allocate(abutl(mxmat,mzchf),stat=ierr)
c       else
c        deallocate(abutv,stat=ierr)
c        if(ierr.ne.0)stop'failure to deallocate abutv'
c        allocate(abutv(mxmat,mzchf)) 
c       endif
c      ENDIF
C
C
C     NOTERM=MIN(NOT1,NRANG2) -- NOT1 RESET IN RSCT, HENCE
C
      NOTERM = NRANG2         !!!! CPB 2004
C      if(iam.eq.0) WRITE (IWRITE,3000)
      KOUNT = 0
      LTEST = 1
      LRECW3 = NOTERM*NOTERM*2 + 1
C
C-----DETERMINE THE NUMBER OF DIPOLE MATRIX ELEMENT SETS
C
      INAST=2             ! Rigged for only 2 symmetries CPB
C      DO 20 JN = 2,INAST
      JN=2
C
       IIII=2*IPAIRCOUNT
C
C-----FINAL STATE DATA
        IG =    IDIPSYM(5,IORDERDIP(IIII,2))
        JPTY =  IDIPSYM(3,IORDERDIP(IIII,2))
        ITEMP = ABS(IG)
        JLRGL = IDIPSYM(2,IORDERDIP(IIII,2))
        JSPN =  IDIPSYM(1,IORDERDIP(IIII,2))
        IF (JSPN.EQ.0) LTEST = 2
        IN=1
C        DO 10 IN = 1,JN - 1
C
       IIII=((2*IPAIRCOUNT)-1) 
C
C-----INITIAL STATE DATA
          IG = IDIPSYM(5,IORDERDIP(IIII,2))
          IPTY = IDIPSYM(3,IORDERDIP(IIII,2))
          ITEMP = ABS(IG)
          ILRGL = IDIPSYM(2,IORDERDIP(IIII,2))
          ISPN = IDIPSYM(1,IORDERDIP(IIII,2))
C
C-----CHECK ELECTRIC DIPOLE SELECTION RULES ARE SATISFIED
C
C          IF (ABS(JLRGL-ILRGL).GT.LTEST .OR.
C     A        (JLRGL+ILRGL).LT.LTEST) GOTO 10
C          IF (ISPN.NE.JSPN .OR. JPTY.EQ.IPTY) GOTO 10
          IF (KOUNT.GE.99) THEN
            if(iam.eq.0) WRITE (IWRITE,3010)
            GOTO 30
C
          ENDIF
C
C          KOUNT = KOUNT + 1
          KOUNT =1
          IMEM(1,KOUNT) = IN
          IMEM(2,KOUNT) = JN
          NCFGP = ILSPI(2,JN) - NOTERM*ILSPI(3,JN)
          LRECW3 = MAX (LRECW3,NCFGP*NOTERM*2+1)
   10   CONTINUE
   20 CONTINUE
C
C-----WRITE THIS BASIC INFORMATION TO OUTPUT FILE
C
   30 CONTINUE
C
C
C
      IF (nosofdipolepairs.EQ.0) WRITE (IWRITE,3020)
      DO 40 IH = 1,KOUNT
C        ITEMP = ILSPI(1,IMEM(1,IH))
        IIII=(2*IPAIRCOUNT-1) 
        IPTY = IDIPSYM(3,IORDERDIP(IIII,2))
C        IF (ITEMP.LT.0) IPTY = 1
C        ITEMP = ABS(ITEMP)
        ILRGL = IDIPSYM(2,IORDERDIP(IIII,2))
        ISPN = IDIPSYM(1,IORDERDIP(IIII,2))
C        ITEMP = ILSPI(1,IMEM(2,IH))
        IIII=(2*IPAIRCOUNT)    
        JPTY = IDIPSYM(3,IORDERDIP(IIII,2))
C        IF (ITEMP.LT.0) JPTY = 1
C        ITEMP = ABS(ITEMP)
        JLRGL = IDIPSYM(2,IORDERDIP(IIII,2))
        JSPN =  IDIPSYM(1,IORDERDIP(IIII,2))
C        if(iam.eq.0) WRITE (ITAPE4) ISPN,ILRGL,IPTY,JSPN,JLRGL,JPTY
C        if(iam.eq.0) WRITE (6,*)ISPN,ILRGL,IPTY,JSPN,JLRGL,JPTY
C     A  IPAIRCOUNT
   40 CONTINUE
C OUT ENDFILE(ITAPE4)
C      if((iam.eq.0).and.(IPAIRCOUNT.eq.nosofdipolepairs))CLOSE (ITAPE4)
C
C     Since D00, is not written to in a single write, then closed 
C     ITAPE5 must be introduced
C
C
C --- *=*=*=*=*=*=*=*
      ITAPE = ITAPE1
      REWIND ITAPE
       call comm_barriersub()
C      if(iam.eq.0)write(6,*)'get here in dmat',KOUNT
C
C      THE DIPOLE MATRIX ELEMENTS ARE READ FROM THE STG2 OUTPUT FILE
C
      DO 120 IH = 1,KOUNT
C
C-----DEFINE INITIAL STATE DATA
C
        IIII=((2*IPAIRCOUNT)-1)
C        
        IN = IMEM(1,IH)
        ILRGL = idipsym(2,iorderdip(IIII,2)) 
        MMNP2 = idipsym(5,iorderdip(IIII,2))
        MCHAN = idipsym(6,iorderdip(IIII,2))
C
C-----DEFINE FINAL STATE DATA
C
        IIII=(2*IPAIRCOUNT)
C
        JN = IMEM(2,IH)
C        JPTY = 0
C        ITEMP = ILSPI(1,JN)
        IF (ITEMP.LT.0) JPTY = 1
C        ITEMP = ABS(ITEMP)
C        JLRGL = MOD(ITEMP,100) - 1
C        JSPN = ITEMP/100
        MNP2 = idipsym(5,iorderdip(IIII,2))
        ILSPI(2,JN) = MNP2
        NCHAN = idipsym(6,iorderdip(IIII,2))
        ILSPI(3,JN)=NCHAN
C
C       The next section deals with symmetries that were already 
C       diagonalised, and therfore have to be constructed from 
C       earlier dipole pairs 
C
C       ie they had no  READ2()
C
C       CPB TO DO : get rid of all N+1 parts of the H.DAT
C                   keep just READ1
C
C
        IIII=((2*IPAIRCOUNT)-1)       !didn't read H.DAT for sym1
C        if(iam.eq.0)write(6,*)IIII,iorderdip(IIII,2),IPAIRCOUNT
        IF(IIII.gt.iorderdip(IIII,2))then
        if(iam.eq.0)then
      write(88,*)'                                                    '
C      write(88,*)'Construct DMAT2 input from sym ',iorderdip(IIII,2)
      write(88,*)'                                                    '
      write(88,*)'                                                    ' 
        endif
        ISPN  = idipsym(1,iorderdip(IIII,2))
        ILRGL = idipsym(2,iorderdip(IIII,2)) 
        MMNP2 = idipsym(5,iorderdip(IIII,2))
        ITEMP = ILSPI(1,IMEM(1,IH))
        IPTY =  idipsym(3,iorderdip(IIII,2)) 
        IF (ITEMP.LT.0) IPTY = 1
        ILSPI(2,IN)=MMNP2
        MCHAN = idipsym(6,iorderdip(IIII,2))
        ILSPI(3,IN)=MCHAN
        ENDIF
C
        IIII=(2*IPAIRCOUNT)
C
        IF(IIII.gt.iorderdip(IIII,2))then
        if(iam.eq.0)then
      write(6,*)'                                                      '
C      write(6,*)'Construct DMAT2 input from sym ',iorderdip(IIII,2)
      write(6,*)'                                                      '
      write(6,*)'                                                      '
        endif
        MNP2 =  idipsym(5,iorderdip(IIII,2))
        ILSPI(2,JN)=MNP2
        NCHAN = idipsym(6,iorderdip(IIII,2))
        ILSPI(3,JN)=NCHAN
        ITEMP = idipsym(4,iorderdip(IIII,2)) 
        JPTY = idipsym(3,iorderdip(IIII,2)) 
        JSPN = idipsym(1,iorderdip(IIII,2))
        JLRGL= idipsym(2,iorderdip(IIII,2))
        ENDIF
C
C
C
C      WRITE TO OUTPUT FILE DATA DEFINING THE S L PI OR J PI
C      COMBINATION SATISFYING THE DIPOLE SELECTION RULES
C
C
       if(iam.eq.0)then
C
	FILNAM = 'D'//NUM(IPAIRCOUNT/10)//NUM(MOD(IPAIRCOUNT,10))
C
C
        OPEN (ITAPE5,FILE=FILNAM,STATUS='UNKNOWN',FORM='UNFORMATTED')
        WRITE (ITAPE5) NOTERM,MNP2,NCHAN,JLRGL,JPTY,JSPN,MMNP2,MCHAN,
     A    ILRGL
        endif
C
C
        IIII=(2*IPAIRCOUNT)
        if(iam.eq.0)write(88,*)(iorderdip(IIII,II),II=1,2)
        IIII=(2*IPAIRCOUNT-1)
        if(iam.eq.0)write(88,*)(iorderdip(IIII,II),II=1,2)
C      if(masteriam.eq.0) WRITE (ITAPE4) ISPN,ILRGL,IPTY,JSPN,JLRGL,JPTY
        if(iam.eq.0) WRITE (88,*)ISPN,ILRGL,IPTY,JSPN,JLRGL,JPTY

C
C        if(iam.eq.0)WRITE(88,*)NOTERM,MNP2,NCHAN,JLRGL,JPTY,JSPN,MMNP2,
C     A    MCHAN,ILRGL
C
C ---- CALCULATE AND WRITE OUT DIPOLE MATRIX ELEMENTS
C
C      CALL DMAT2, WITH SCRATCH SPACE IN /MEMORY/, IF SUFFICIENT.
C      IF EIGENVECTORS ALREADY IN MEMORY, SET IV AND JV TO POINT TO THEM
C
C
        IV = ABS(ILSPI(4,IN)) + 1
        JV = ABS(ILSPI(4,JN)) + 1
        KV = MEM1 + 2*MMNP2*MNP2
        IF (ILSPI(4,IN).GT.0) THEN
          IV = KV + 1
          KV = KV + MMNP2**2
        ENDIF
C
        IF (ILSPI(4,JN).GT.0) THEN
          JV = KV + 1
          KV = KV + MNP2**2
        ENDIF
C
C        IF (KV.LE.MXMEM) THEN
          ML = MEM1 + 1
          MV = ML + MMNP2*MNP2
         if(iam.eq.0) write(88,*)'         CALLING MODIFIED DMAT2'
         if(iam.eq.0)write(88,*)IN,JN,NRANG2,NOTERM,MMNP2,MNP2,ML,MV
         call comm_barriersub()
C         call comm_barrier()
C         call comm_finalize()
C         stop
        CALL DMAT2(IN,JN,NRANG2,NOTERM,MMNP2,MNP2,NCHAN,MCHAN)
C         ELSE
C	 write(88,*)'better not get here'
C	ENDIF  
C         write(88,*)'leaving DMAT2'
C
C
C      /MEMORY/ INSUFFICIENT, CALL DMAT1, WITH SCRATCH SPACE IN /MATRIX/
C
C
C
C          M = 1 + (KV-1)/1000000
C          WRITE (IWRITE,*) ' *** TO IMPROVE EFFICIENCY, INCREASE ',
C     A      'MZMEG TO ',M
C          CALL DMAT1(IN,JN,NRANG2,NOTERM,MMNP2,MNP2,LRECW3)
C
C        No significant nested do loops from here on out,just let 
C        processor 0 handle things 
C
C-----WRITE OUT BLOCKS INVOLVING BUTTLE CORRECTIONS, REORDERING CHANNELS
C
C
C        IPAIRCOUNT=IPAIRCOUNT+1 
C
        if(ndip.eq.1)then
        allocate(abutl(mnp2,mchan))
        else
        allocate(abutv(mnp2,mchan))
        endif 
c
        if(iam.eq.0)then
         rewind(19)
         if(ndip.eq.1)then
           DO  M=1,MCHAN
            DO NP=1,MNP2
             READ(19) ABUTL(NP,M)
            ENDDO
           ENDDO
         else
           DO  M=1,MCHAN
            DO NP=1,MNP2
             READ(19) ABUTV(NP,M)
            ENDDO
           ENDDO
         endif
         close(19)
C
C
C
C        write(88,*)IN,MCHAN,MNP2
C 
        if(ndip.eq.1)then 
        WRITE (ITAPE5) ((ABUTL(I,IORDER(J,IN)),J=1,MCHAN),I=1,MNP2),
     A    ((dumm,J=1,MCHAN),I=1,MNP2)
        WRITE (ITAPE5) ((BBUTL(I,IORDER(J,JN)),J=1,NCHAN),I=1,MMNP2),
     A    ((dumm,J=1,NCHAN),I=1,MMNP2)
         else
        WRITE (ITAPE5) ((dumm,J=1,MCHAN),I=1,MNP2),
     A    ((ABUTV(I,IORDER(J,IN)),J=1,MCHAN),I=1,MNP2)
        WRITE (ITAPE5) ((dumm,J=1,NCHAN),I=1,MMNP2),
     A    ((BBUTV(I,IORDER(J,JN)),J=1,NCHAN),I=1,MMNP2)
         endif	 
C
C      READ AND WRITE OUT BUTTLE-BUTTLE ELEMENTS: BBUT(NCHAN,MCHAN)
C
        if(ndip.eq.1)then
        READ (ITAPE) ((BBUTL(I,J),J=1,NCHAN),I=1,MCHAN)
        READ (ITAPE) ((dumm,J=1,NCHAN),I=1,MCHAN)
        WRITE (ITAPE5) ((BBUTL(IORDER(I,IN),IORDER(J,JN)),J=1,NCHAN),
     A    I=1,MCHAN), ((dumm,J=1,NCHAN),I=1,
     B    MCHAN)
        else
        READ (ITAPE) ((dumm,J=1,NCHAN),I=1,MCHAN)
        READ (ITAPE) ((BBUTV(I,J),J=1,NCHAN),I=1,MCHAN)
        WRITE (ITAPE5) ((dumm,J=1,NCHAN),
     A    I=1,MCHAN), ((BBUTV(IORDER(I,IN),IORDER(J,JN)),J=1,NCHAN),I=1,
     B    MCHAN)
        endif
C
        IF(NDIP.eq.1)THEN
        if(allocated(bbutl))deallocate(BBUTL)
        else
        if(allocated(bbutv))deallocate(BBUTV)
        ENDIF	
C
C     READ IN THE CLEBSCH GORDAN COEFFICIENTS FOR THE POLARIZABILITY
C     CALCULATION, WRITE THESE TO THE OUTPUT FILE:
C
        ALLOCATE(CGC(MZLR2))
C
        READ (ITAPE) MAXM1, (CGC(M),M=1,MAXM1)
        WRITE (ITAPE5) MAXM1, (CGC(M),M=1,MAXM1)
C
        DEALLOCATE(CGC)       
C
C-----READ IN AND WRITE OUT THE ANGULAR INTEGRALS NEEDED FOR OUTER
C-----REGION INTEGRATION, REORDERING CHANNELS.
C
        ALLOCATE(AC(NCHAN,MCHAN))
        ALLOCATE(BLC(NCHAN,MCHAN))
        ALLOCATE(BVC(NCHAN,MCHAN))
C
        READ (ITAPE) ((AC(I,J),J=1,MCHAN),I=1,NCHAN)
        READ (ITAPE) ((BLC(I,J),J=1,MCHAN),I=1,NCHAN)
        READ (ITAPE) ((BVC(I,J),J=1,MCHAN),I=1,NCHAN)
        WRITE (ITAPE5) ((AC(IORDER(I,JN),IORDER(J,IN)),J=1,MCHAN),I=1,
     A    NCHAN), ((BLC(IORDER(I,JN),IORDER(J,IN)),J=1,MCHAN),I=1,
     B    NCHAN), ((BVC(IORDER(I,JN),IORDER(J,IN)),J=1,MCHAN),I=1,NCHAN)
C
        IF (NBUG7.EQ.0) GOTO 80
        WRITE (6,3050)
        DO 50 I = 1,NCHAN
          II = IORDER(I,JN)
          WRITE (6,3030) II, (AC(II,IORDER(J,IN)),J=1,MCHAN)
   50   CONTINUE
        WRITE (6,3060)
        DO 60 I = 1,NCHAN
          II = IORDER(I,JN)
          WRITE (6,3030) II, (BLC(II,IORDER(J,IN)),J=1,MCHAN)
   60   CONTINUE
        WRITE (6,3070)
        DO 70 I = 1,NCHAN
          II = IORDER(I,JN)
          WRITE (6,3030) II, (BVC(II,IORDER(J,IN)),J=1,MCHAN)
   70   CONTINUE
C
C
   80   CONTINUE
C
        DEALLOCATE(AC,BLC,BVC)
C
   81   IF(ISPN.EQ.0)THEN
          ALLOCATE(AC(NCHAN,MCHAN)) 
          READ (ITAPE,END=100,ERR=90)((AC(I,J),J=1,MCHAN),I=1,NCHAN)
          WRITE (ITAPE5)((AC(IORDER(I,JN),IORDER(J,IN)),J=1,MCHAN),
     A    I=1,NCHAN)
C
C         
          GO TO 110
   90     BACKSPACE (ITAPE)
  100     WRITE (88,3080)
        ENDIF
C
         endif      !    long iam=0 statement     CPB
C
C OUT ENDFILE(ITAPE4)
  110   CLOSE (ITAPE5)
          if((ispn.eq.0).and.(iam.eq.0))deallocate(ac,stat=ierr)
          if(ierr.ne.0)write(0,*)'deallocate for ac failed'
          call comm_barriersub()
C
C --- *=*=*=*=*=*=*=*
C
  120 CONTINUE
C
 3000 FORMAT (/35X,'SUBROUTINE DMAT'/35X,15 ('-'))
 3010 FORMAT (' IT IS IMPOSSIBLE TO CREATE D-FILES BEYOND D99.')
 3020 FORMAT (/' DMAT HAS NOT ENCOUNTERED ANY PERMITTED TRANSITION!')
 3030 FORMAT (I5, (T6,5E15.7))
 3040 FORMAT (' INITIAL STATE NUMBER',I3,6X,'FINAL STATE NUMBER',I3)
 3050 FORMAT (/' THE A COEFFICIENTS ARE')
 3060 FORMAT (/' THE BL COEFFICIENTS ARE')
 3070 FORMAT (/' THE BV COEFFICIENTS ARE')
 3080 FORMAT (/' NO ASYMPTOTIC BP-VELOCITY ARRAY FOUND - SKIPPING')
      END
      SUBROUTINE DMAT2(IN,JN,NRANG2,NOTERM,MMNP2,MNP2,NCHAN,MCHAN)
      use param1
      use eigvec
      use comm_interface
      use symrec
      use matrix
      IMPLICIT REAL*8 (A-H,O-Z)
      include 'mpif.h'
      real*8,allocatable :: DUM1(:),DUM2(:)
      real*8,allocatable :: DML(:,:),DMV(:,:),DUM3(:,:)
      real*8,allocatable :: DMLL(:,:),DMLL2(:),DMLZ(:,:),DMLZ2(:)
      integer,dimension(:) :: isizework(0:nproc-1),nproctotal(0:nproc-1)
      integer :: status(MPI_STATUS_SIZE)
      integer,allocatable :: KKSTART(:),KKFINAL(:),TMPA(:)
C
      CHARACTER*1 NUM(0:9)
      DATA NUM/'0','1','2','3','4','5','6','7','8','9'/
C
      data ZERO/0.0d0/
      data ONE/1.0d0/
      data ICOUNT/0/
      SAVE ICOUNT 
C
C
C
C-----------------------------------------------------------------------
C
C
C      PARAMETER (MXMAT=MZCHF*MZNR2+MZNC2)
C
c      DIMENSION ABUTL(MXMAT,MZCHF),ABUTV(MXMAT,MZCHF),
c     A          BBUTL(MXMAT,MZCHF),BBUTV(MXMAT,MZCHF)
C      DIMENSION DML(MNP2,MMNP2)
C      DIMENSION DUM1(MXMAT),DUM2(MXMAT)
C
      COMMON /DISTAP/IDISC1,IDISC2,IDISC3,IDISC4,ITAPE1,ITAPE2,ITAPE3,
     A       ITAPE4,ITAPE5
      COMMON /INFORM/IREAD,IWRITE,IPUNCH
      COMMON /NBUG/NBUG1,NBUG2,NBUG3,NBUG4,NBUG5,NBUG6,NBUG7,NBUG8,NBUG9
      COMMON /RECORD/ILSPI(4,MZSLP),IORDER(MZCHF,MZSLP)
C      COMMON /SPACES/XX(MZNR2,MXMAT)
C      COMMON /EIGVEC/VI(MMNP2,MMNP2),VJ(MNP2,MNP2)
C-----------------------------------------------------------------------
C      MCHAN = ILSPI(3,IN)
C      NCHAN = ILSPI(3,JN)
C      MMNP2 = ILSPI(2,IN)
C       MNP2 = ILSPI(2,JN)
C
C      write(6,*)'MXMAT=',MXMAT,MASTERIAM,IAM

      ICOUNT=ICOUNT+1
      if(iam.eq.0)then
      write(88,*)'MMNP2 =',MMNP2,'MNP2 =',MNP2,'NCHAN=',NCHAN,'MCHAN=',
     X           MCHAN
      endif
C
C
      IDUM=MIN(MCHAN,NCHAN)
C
C
C
      IF(NPROC.gt.IDUM)then
       if(iam.eq.0)write(88,*)'                                       ' 
       if(iam.eq.0)write(88,*)' Small Number of Channels Case         '
       if(iam.eq.0)write(88,*)'                                       '
       if(iam.eq.0)write(0,*)'Small channels use DISTRIBUTE file      '
       stop 
      ENDIF 
C
C
      IF(NPROC.gt.IDUM)then
          NPROCSIZE=IDUM
      else
          NPROCSIZE=NPROC
      ENDIF
C
C
C
C
      ITAPE = ITAPE1
C
      IDUM=MAX(MMNP2,MNP2)   ! set DUM array to the largest dim of either sym
      allocate(DUM1(IDUM),DUM2(IDUM),stat=ierr1)
      if(ierr1.ne.0)stop 'allocation fails for DUM1,DUM2  - DMAT2  '
C      NDIP=1
      if(NDIP.eq.1)then
      allocate(DML(MNP2,MMNP2),stat=ierr1)
      if(ierr1.ne.0)stop 'allocation fails for DML  - DMAT2  '
      else
      allocate(DMV(MNP2,MMNP2),stat=ierr1)
      if(ierr1.ne.0)stop 'allocation fails for DMV  - DMAT2  '
      endif
C
       IF((IAM+1).gt.NPROCSIZE)GOTO 556
C
C
C
C
C
C   read in dipole matrix DML,DMV from stg2
C   read in RHS Buttle correction into ABUTL,ABUTV
C
      N2 = 0
C      if(iam.eq.0)write(6,*)'In dmat2 ',MCHAN,NCHAN,NRANG2,ITAPE,icolour
      DO 20 NCH = 1,NCHAN
        N1 = N2 + 1
        N22 = N2 + NRANG2
        M2 = 0
        DO 10 MCH = 1,MCHAN
          if(NDIP.eq.1)then 
          READ (ITAPE) ((DML(N,M),N=N1,N22),M=M2+1,M2+NRANG2)
          READ (ITAPE)  
          else
          READ (ITAPE) 
          READ (ITAPE) ((DMV(N,M),N=N1,N22),M=M2+1,M2+NRANG2)
          endif
          M2 = M2 + NOTERM
   10   CONTINUE
        IF (M2.LT.MMNP2) THEN
          if(NDIP.eq.1)then
          READ (ITAPE) ((DML(N,M),N=N1,N22),M=M2+1,MMNP2)
          READ (ITAPE) 
          else
          READ (ITAPE) 
          READ (ITAPE) ((DMV(N,M),N=N1,N22),M=M2+1,MMNP2)
          endif
        ENDIF
C
        if(ndip.eq.1)then
        READ (ITAPE) ((ABUTL(N,M),N=N1,N22),M=1,MCHAN)
        READ (ITAPE) ((dumm,N=N1,N22),M=1,MCHAN)
	else
        READ (ITAPE) ((dumm,N=N1,N22),M=1,MCHAN)
        READ (ITAPE) ((ABUTV(N,M),N=N1,N22),M=1,MCHAN)
	endif	
        N2 = N2 + NOTERM
   20 CONTINUE
C        if(iam.eq.0)write(6,*)'In dmat2 : get past 20 read ',icolour
      IF (N2.LT.MNP2) THEN
        N1 = N2 + 1
        M2 = 0
        DO 30 MCH = 1,MCHAN
        if(NDIP.eq.1)then
          READ (ITAPE) ((DML(N,M),N=N1,MNP2),M=M2+1,M2+NRANG2)
          READ (ITAPE) 
         else
          READ (ITAPE) 
          READ (ITAPE) ((DMV(N,M),N=N1,MNP2),M=M2+1,M2+NRANG2)
         endif
          M2 = M2 + NOTERM
   30   CONTINUE
C       if(iam.eq.0)write(6,*)'In dmat2 : get past 30 read ',icolour
        IF (M2.LT.MMNP2) THEN
          NDIMEN = NRANG2
          NTIMES = (MNP2-N2-1)/NDIMEN + 1
          DO 40 NT = 1,NTIMES
            N22 = MIN(N2+NDIMEN,MNP2)
        if(NDIP.eq.1)then
            READ (ITAPE) ((DML(N,M),N=N2+1,N22),M=M2+1,MMNP2)
            READ (ITAPE) 
        else
            READ (ITAPE) 
            READ (ITAPE) ((DMV(N,M),N=N2+1,N22),M=M2+1,MMNP2)
        endif
            N2 = N22
   40     CONTINUE
C       if(iam.eq.0) write(6,*)'In dmat2 : get past 40 read ',icolour
        ENDIF
C
        if(ndip.eq.1)then
        READ (ITAPE) ((ABUTL(N,M),N=N1,MNP2),M=1,MCHAN)
        READ (ITAPE) ((dumm,N=N1,MNP2),M=1,MCHAN)
	else
        READ (ITAPE) ((dumm,N=N1,MNP2),M=1,MCHAN)
        READ (ITAPE) ((ABUTV(N,M),N=N1,MNP2),M=1,MCHAN)
	endif	
      ENDIF
C
C
C   multiply RHS V by dipole matrix, overwriting DML and DMV
C
      ITMP1=MMNP2/NPROCSIZE
      ITMP2=MOD(MMNP2,NPROCSIZE)
C
C
C
       do I=0,nprocsize-1
        isizework(I)=ITMP1
        if(I.le.(ITMP2-1))then
        isizework(I)=isizework(I)+1
        endif
C        write(6,*)'nsize =',isizework(I),I,MMNP2
       enddo
C
C      Upper limit of block
C
C
        if(iam.ge.0)then
        isize=0
        do I=0,iam
        isize=isize+isizework(I)
        enddo
        endif
C
C       Lower limit of block
C
       if(iam.eq.0)then
       nprocwork=1
       else
       nprocwork=1+iam*ITMP1
                 if(iam.le.(ITMP2-1))then
                 nprocwork=nprocwork+iam
                 else
                 nprocwork=nprocwork+ITMP2 
                 endif
       endif
C
C
C
      allocate(DMLL(MNP2,nprocwork:isize),stat=ierr1)
      if(ierr1.ne.0)stop 'allocation fails for DMLL  - DMAT2  '
      if(iam.eq.0)then
      allocate(DMLZ(MNP2,ITMP1+1),stat=ierr1)
      if(ierr1.ne.0)stop 'allocation fails for DMLZ  - DMAT2  '
      endif
C
C      write(6,*)'In dmat2 : get past DMLL and DMLZ'
C
C
      DO 68 M = nprocwork,isize
       DO 69 NP = 1,MNP2
        DMLL(NP,M) = 0.0D0
  69   continue
  68   continue
C
       KKCOUNT=0
C
       KKSPLIT=NRANG2
C
       kkmod=mnp2/kksplit
C
       allocate(kkstart(0:kksplit-1))
       allocate(kkfinal(0:kksplit-1))
        do JJ=0,KKSPLIT-1
        KKSTART(JJ)=1+kkmod*JJ
        KKFINAL(JJ)=kkmod*(JJ+1)
        enddo
        KKFINAL(KKSPLIT-1)=MNP2
C
C
       if(mnp2.lt.100)then
       kksplit=1
       kkstart(0)=1
       kkfinal(0)=mnp2
       endif 
C
C
      OPEN (UNIT=11,FILE=sym2,STATUS='OLD',FORM='UNFORMATTED')
C
C     VIJ is in vertical columns .... Im doing the first 
C     matrix multiplication in NRANG2 strips
C
C     ****************************************************************
C
C      FIRST MATRIX MULTIPLY  D * VJ 
C
C     ***************************************************************** 
C
  70   continue
C
      iistart=kkstart(kkcount)
      iifinal=kkfinal(kkcount)
C
C
C
         allocate(VJ(MNP2,iistart:iifinal),stat=ierr1)
       if(ierr1.ne.0)stop 'allocation fails for VJ  - DMAT2  '
         DO 212 NO = iistart,iifinal 
         READ(11)(VJ(N,NO),N=1,MNP2)
  212    CONTINUE
C
C
C
C
      if(ndip.eq.1)then
      DO 110 M = nprocwork,isize
        DO 90 N = iistart,iifinal 
          DCONA=DML(N,M)
          DO 80 NP = 1,MNP2
          DMLL(NP,M) = DMLL(NP,M) + DCONA*VJ(NP,N)
   80     CONTINUE
   90   CONTINUE
  110 CONTINUE
      else
C
      DO 10110 M = nprocwork,isize
        DO 1090 N = iistart,iifinal 
          DCONA=DMV(N,M)
          DO 1080 NP = 1,MNP2
          DMLL(NP,M) = DMLL(NP,M) + DCONA*VJ(NP,N)
 1080     CONTINUE
 1090   CONTINUE
10110 CONTINUE
       endif
C
        if(allocated(vj))deallocate(VJ)
C
C      DECISION Made to deallocate and VJ here, and re-read for the
C      dipole matrix multiply VJ*abut(v/l) later on .... solely a 
C      a memory concern
C
C
       kkcount=kkcount+1
       if(kkcount.eq.(kksplit))then
       continue
       else
       goto 70
       endif 
C
C
       if((iam.eq.0).and.(ndip.eq.1))then
       DO 111 M = nprocwork,isize
       DO 112 NP = 1,MNP2
        DML(NP,M) = DMLL(NP,M) 
  112  continue
  111  continue 
       if(allocated(dmll))deallocate(DMLL)
       endif
C
       if((iam.eq.0).and.(ndip.ne.1))then
       DO 10111 M = nprocwork,isize
       DO 10112 NP = 1,MNP2
        DMV(NP,M) = DMLL(NP,M)
10112  continue
10111  continue
       if(allocated(dmll))deallocate(DMLL)
       endif
C
C
        itagwmat=43
        itagpoint1=44
        if (iam.ne.0) then
           CALL MPI_SEND(nprocwork,1,MPI_INTEGER,0,itagpoint1,
     *                   new_comm,ierr)
C           write(6,*)'sending nprocwork',nprocwork,iam
           CALL MPI_SEND(DMLL,isizework(iam)*MNP2,MPI_REAL8,0,itagwmat,
     *                   new_comm,ierr)
           if (ierr.ne.0) write(6,*)'MPI_SEND: iam, ierr = ', iam, ierr
        else
c....... Receive the buffers from other processors, and add to global DM? file 
           do J = 1, nprocsize-1
              CALL MPI_RECV(ibuff,1,MPI_INTEGER,J,
     *                      itagpoint1,new_comm,status,ierr)
            if (ierr.ne.0) write(6,*)'MPI_RECV: iam, ierr = ', iam, ierr
              CALL MPI_RECV(DMLZ,isizework(J)*MNP2,MPI_REAL8,J,itagwmat,
     *                      new_comm,status,ierr)
            if (ierr.ne.0) write(6,*)'MPI_RECV: iam, ierr = ', iam, ierr
C            write(6,*)'rec ibuff=',ibuff,J,iam,isizework(J)
                ibuff=ibuff-1
              if(NDIP.eq.1)then
                do M=1,isizework(J)
                  ibuff=ibuff+1
                  do L=1,MNP2
                 DML(L,ibuff) = DMLZ(L,M)
                enddo
              enddo
              else
                do M=1,isizework(J)
                  ibuff=ibuff+1
                  do L=1,MNP2
                 DMV(L,ibuff) = DMLZ(L,M)
                enddo
              enddo
              endif
           enddo
        endif
C
C
C       if(iam.eq.0)write(0,*)'after first mpi collect',icolour
C       call flush(0)
C
C     Overcomes the small buffer size on small clusters
C
            if(iam.eq.0)then
            if(allocated(dmlz))deallocate(dmlz)
            endif
C
C
            if(iam.eq.0)then
C
C
            ii1=icolour/10
            ii2=icolour-10*ii1
C
C
       OPEN(17,file='DSCRATCH_'//NUM(ii1)//NUM(ii2),status='unknown',
     X      form='unformatted',access='STREAM')
            if(ndip.eq.1)then
            do K=1,MMNP2
            write(17)(DML(I,K),I=1,MNP2)
            enddo
            if(allocated(dml))deallocate(dml)
            else
            do K=1,MMNP2
            write(17)(DMV(I,K),I=1,MNP2)
            enddo
            if(allocated(dmv))deallocate(dmv)
            endif
C
            close(17)
            endif
            call comm_barriersub()
C
  556       continue
            if(iam.eq.0)write(88,*)'FINISHED WRITING DSCRATCH'
            call comm_barriersub()
            rewind(11)
       
C            if(iam.eq.0)write(6,*)'middle of DMAT2'
            IF((IAM+1).gt.NPROCSIZE)stop'use DISTRIBUTED file'

C
C
C     START of the first Buttle matrix multiply
C
C
      if(iam.eq.0)then
C
      allocate(DUM3(MNP2,MCHAN),stat=ierr)
      if(ierr.ne.0)stop 'allocation fails for dum3'
C
        do ji=1,mchan
         do jk=1,mnp2
          dum3(jk,ji)=0.0d0
         enddo
        enddo
C 
        kkcount=0             
  75     continue
C
C
       iistart=kkstart(kkcount)
       iifinal=kkfinal(kkcount)
C
C            
       allocate(VJ(MNP2,iistart:iifinal),stat=ierr1)
       if(ierr1.ne.0)stop 'allocation fails for VJ  - DMAT2  '
         DO 214 NO = iistart,iifinal
         READ(11)(VJ(N,NO),N=1,MNP2)
  214    CONTINUE
C

      if(ndip.eq.1)then
C
      DO 4001 J = 1,MCHAN
        DO 3001 K = iistart,iifinal
          DCONA=ABUTL(K,J)
          DO 2001 I = 1,MNP2
            DUM3(I,J) = VJ(I,K)*DCONA + DUM3(I,J)
 2001     CONTINUE
 3001   CONTINUE
 4001  CONTINUE
C
      else
      DO 4002 J = 1,MCHAN
        DO 3002 K = iistart,iifinal
          DCONA=ABUTV(K,J)
          DO 2002 I = 1,MNP2
            DUM3(I,J) = VJ(I,K)*DCONA + DUM3(I,J)
 2002     CONTINUE
 3002   CONTINUE
 4002  CONTINUE
      endif
C
      if(allocated(vj))deallocate(vj,stat=ierr)
      if(ierr.ne.0)stop 'allocation fails for vj'
C
       kkcount=kkcount+1
       if(kkcount.eq.(kksplit))then
       if(allocated(kkstart))deallocate(kkstart,kkfinal)
       continue
       else
       goto 75
       endif
C
      close(11)             
C
      if(ndip.eq.1)then
      DO 160 M=1,MCHAN
      DO 150 NP=1,MNP2
       ABUTL(NP,M)=DUM3(NP,M)
 150  CONTINUE
 160  CONTINUE
C
      ii1=icolour/10
      ii2=icolour-10*ii1
      OPEN(19,file='DSCRATCH2_'//NUM(ii1)//NUM(ii2),
     X      form='unformatted',status='unknown',access='stream')
       DO 161 M=1,MCHAN
        DO 151 NP=1,MNP2
        WRITE(19)ABUTL(NP,M)
 151   CONTINUE
 161   CONTINUE  
C
      if(allocated(dum3))deallocate(dum3,stat=ierr)
      if(ierr.ne.0)stop'dealloc for dum3 fails'
      allocate(dml(MNP2,MMNP2),stat=ierr)
      if(ierr.ne.0)stop 'allocation fails for dml' 
C
C
      else
      DO 10160 M=1,MCHAN
      DO 10150 NP=1,MNP2
       ABUTV(NP,M)=DUM3(NP,M)
10150  CONTINUE
10160  CONTINUE 
C

       ii1=icolour/10
       ii2=icolour-10*ii1
       OPEN(19,file='DSCRATCH2_'//NUM(ii1)//NUM(ii2),
     X        form='unformatted',status='unknown',access='stream')
       DO 10161 M=1,MCHAN
        DO 10151 NP=1,MNP2
        WRITE(19)ABUTV(NP,M)
10151   CONTINUE
10161   CONTINUE  
C
C
      if(allocated(dum3))deallocate(dum3,stat=ierr)
      if(ierr.ne.0)stop'dealloc faor dum 3 fails'
      allocate(dmv(MNP2,MMNP2),stat=ierr)
      if(ierr.ne.0)stop 'allocation fails for dmv'
C
C 
      endif
      endif
C
      if(ndip.eq.1)then
      if(allocated(abutl))deallocate(abutl)
      else
      if(allocated(abutv))deallocate(abutv)
      endif
C
      if(iam.eq.88)write(6,*)'FINISHED WRITING FIRST BUTTLE MULTIPLY'
C
C
C      ONTO the second matrix multiply 'VI^T * DML/V '
C      Again to save memory, i get everybody to read the full DML/v(MMNP2,MMNP2)
C      matrix, but then reshape to DML(nprocwork:isize,MMN2P) before I read 
C      in V^T
C
        ii1=icolour/10
        ii2=icolour-10*ii1
        OPEN(18,file='DSCRATCH_'//NUM(ii1)//NUM(ii2),status='unknown',
     X       form='unformatted',access='STREAM')
C
C
C
            if(ndip.eq.1)then
            do K=1,MMNP2
            read(18)(DML(I,K),I=1,MNP2)
            enddo
            else
            do K=1,MMNP2
            read(18)(DMV(I,K),I=1,MNP2)
            enddo
            endif      
C
C
C
         ITMP1=MNP2/NPROCSIZE
         ITMP2=MOD(MNP2,NPROCSIZE)
C
C
       do I=0,nprocsize-1
        isizework(I)=ITMP1
        if(I.le.(ITMP2-1))then
        isizework(I)=isizework(I)+1
        endif
C        write(6,*)'nsize =',isizework(I),I,MNP2
       enddo
C
        if(iam.ge.0)then
        isize=0
        do I=0,iam
        isize=isize+isizework(I)
        enddo
        endif
C
       if(iam.eq.0)then
       nprocwork=1
       else
       nprocwork=1+iam*ITMP1
                 if(iam.le.(ITMP2-1))then
                 nprocwork=nprocwork+iam
                 else
                 nprocwork=nprocwork+ITMP2
                 endif
       endif
C
C
C
       allocate(dum3(nprocwork:isize,mmnp2))
        if(ndip.eq.1)then
         do i =1,mmnp2
          do j=nprocwork,isize 
            dum3(j,i)=dml(j,i)
          enddo
          enddo
        else
         do i =1,mmnp2
          do j=nprocwork,isize
           dum3(j,i)=dmv(j,i)
          enddo
         enddo
        endif
C
C
       if(ndip.eq.1)then
       if(allocated(dml))deallocate(dml)
       allocate(dml(nprocwork:isize,mmnp2))
       else
       if(allocated(dmv))deallocate(dmv)
       allocate(dmv(nprocwork:isize,mmnp2))
       endif
C
C
      

        if(ndip.eq.1)then
        do i=1,mmnp2
         do j=nprocwork,isize
          dml(j,i)=dum3(j,i)
         enddo
        enddo
        else
        do i=1,mmnp2
         do j=nprocwork,isize
          dmv(j,i)=dum3(j,i)
         enddo
        enddo
        endif
C

       if(allocated(dum3))deallocate(dum3)
C  
C
      if(iam.eq.0)write(88,*)'AFTER DML reshape'
C
      OPEN (UNIT=8,FILE=sym1,STATUS='OLD',FORM='UNFORMATTED')
         allocate(VI(MMNP2,MMNP2))
         DO 555 NO = 1,MMNP2 
          READ(8)(VI(NO,N),N=1,MMNP2)
  555   CONTINUE
        close(8)
C
C
C
C
C
      if(iam.ne.0)then
      if(allocated(dmll))deallocate(DMLL)
      allocate(DMLL2((ITMP1+2)*MMNP2))
      endif
C
C
      KK=0
C
      if(ndip.eq.1)then
C
      DO 230 NP = nprocwork,isize 
        DO 190 MP = 1,MMNP2
          DUM1(MP) = 0.0D0
  190   CONTINUE
        DO 210 M = 1,MMNP2
          DCON=DML(NP,M)
          DO 200 MP = 1,MMNP2
          DUM1(MP) = DUM1(MP) + VI(MP,M)*DCON
  200     CONTINUE
  210   CONTINUE
        DO 220 MP = 1,MMNP2
          if(iam.ne.0)then
          KK=KK+1
              DMLL2(KK) = DUM1(MP)
          else
              DML(NP,MP) = DUM1(MP)
          endif 
  220   CONTINUE
  230 CONTINUE
        else
      DO 10230 NP = nprocwork,isize
        DO 10190 MP = 1,MMNP2
          DUM1(MP) = 0.0D0
10190   CONTINUE
        DO 10210 M = 1,MMNP2
          DCON=DMV(NP,M)
          DO 10200 MP = 1,MMNP2
          DUM1(MP) = DUM1(MP) + VI(MP,M)*DCON
10200     CONTINUE
10210   CONTINUE
        DO 10220 MP = 1,MMNP2
          if(iam.ne.0)then
          KK=KK+1
              DMLL2(KK) = DUM1(MP)
          else
              DMV(NP,MP) = DUM1(MP)
          endif
10220   CONTINUE
10230 CONTINUE
       endif
C
        if(allocated(vi))deallocate(vi)
C
C
        if(iam.eq.0)then
        allocate(DMLZ2((ITMP1+2)*MMNP2))
        endif
C
        if(iam.eq.0)then
        allocate(dum3(1:isize,mmnp2))

        if(ndip.eq.1)then
         do i=1,mmnp2
          do j=1,isize
             dum3(j,i)=dml(j,i)
          enddo 
         enddo
        else
         do i=1,mmnp2
          do j=1,isize    
             dum3(j,i)=dmv(j,i)
          enddo
         enddo
        endif
C
        if(ndip.eq.1)then
        if(allocated(dml))deallocate(dml)
        allocate(dml(mnp2,mmnp2))
        else
        if(allocated(dmv))deallocate(dmv)
        allocate(dmv(mnp2,mmnp2))
        endif
C
        if(ndip.eq.1)then
        do i=1,mmnp2
         do j=1,isize
             dml(j,i)=dum3(j,i)
         enddo
        enddo
        else
        do i=1,mmnp2
         do j=1,isize
             dmv(j,i)=dum3(j,i)
         enddo
        enddo
        endif

        if(allocated(dum3))deallocate(dum3)
C
C
        endif
C
        itagwmat=43
        itagpoint1=44
        itagpoint2=45
        isizew=isize-nprocwork+1
        if (iam.ne.0) then
           CALL MPI_SEND(nprocwork,1,MPI_INTEGER,0,itagpoint1,
     *                   new_comm,ierr)
C           write(6,*)'sending nprocwork',nprocwork,iam
           if (ierr.ne.0) write(6,*)'MPI_SEND: iam, ierr = ', iam, ierr
           CALL MPI_SEND(isizew,1,MPI_INTEGER,0,itagpoint2,
     *                   new_comm,ierr)
           if (ierr.ne.0) write(6,*)'MPI_SEND: iam, ierr = ', iam, ierr
           CALL MPI_SEND(DMLL2,isizew*MMNP2,MPI_REAL8,0,itagwmat,
     *                   new_comm,ierr)
           if (ierr.ne.0) write(6,*)'MPI_SEND: iam, ierr = ', iam, ierr

        else
c....... Receive the buffers from other processors, and add to global DM? file 
           do J = 1, nprocsize-1
              CALL MPI_RECV(ibuff,1,MPI_INTEGER,J,
     *                      itagpoint1,new_comm,status,ierr)
            if (ierr.ne.0) write(6,*)'MPI_RECV: iam, ierr = ', iam, ierr
              CALL MPI_RECV(ibuff2,1,MPI_INTEGER,J,
     *                      itagpoint2,new_comm,status,ierr)
            if (ierr.ne.0) write(6,*)'MPI_RECV: iam, ierr = ', iam, ierr
            CALL MPI_RECV(DMLZ2,isizework(J)*MMNP2,MPI_REAL8,J,itagwmat,
     *                      new_comm,status,ierr)
            if (ierr.ne.0) write(6,*)'MPI_RECV: iam, ierr = ', iam, ierr
C                if(isizework(J).ne.ibuff2)write(isizework(J),ibuff2)
                ibuff=ibuff-1
                KK=0
              if(ndip.eq.1)then
                do M=1,ibuff2
                 ibuff=ibuff+1
                  do L=1,MMNP2
                    KK=KK+1
                 DML(ibuff,L) = DMLZ2(KK)
                enddo
                enddo
              else
                do M=1,ibuff2
                   ibuff=ibuff+1
                  do L=1,MMNP2
                    KK=KK+1
                 DMV(ibuff,L) = DMLZ2(KK)
                  enddo
                enddo
              endif
           enddo
        endif
C
C
        if(iam.eq.0)then
        if(allocated(dmlz2))then
        deallocate(DMLZ2,stat=ierr)
        endif
        if(ierr.ne.0)write(0,*)'failure to deallocate DMLZ2'
        else
        if(allocated(dmll2))then
        deallocate(DMLL2,stat=ierr)
        endif
        if(ierr.ne.0)write(0,*)'failure to deallocate DMLL2'
        endif
C
C
C      WRITE OUT DIPOLE MATRICES
C
        if(iam.eq.0)then
      N2 = 0
      DO 330 IK = 1,1 + (MMNP2-1)/NOTERM
        N1 = N2 + 1
        N2 = MIN(N2+NOTERM,MMNP2)
        M2 = 0
        DO 320 JK = 1,1 + (MNP2-1)/NOTERM
          M1 = M2 + 1
          M2 = MIN(M2+NOTERM,MNP2)
          if(NDIP.ne.1) then
          WRITE (ITAPE5) ((DMV(M,N),M=M1,M2),N=N1,N2),
     A      ((DMV(M,N),M=M1,M2),N=N1,N2)
          else
          WRITE (ITAPE5) ((DML(M,N),M=M1,M2),N=N1,N2),
     A      ((DML(M,N),M=M1,M2),N=N1,N2)
C          write(6,*)M2,N2
          endif
C
C
          NBUG7=0
        IF(NDIP.eq.1)then
          IF (NBUG7.LT.1) GOTO 320
          WRITE (77,*) ' LENGTH ',IK,JK
          DO 300 N = N1,N2
            WRITE (77,3000) (DML(M,N),M=M1,M2)
  300     CONTINUE
        ENDIF
        IF(NDIP.ne.1)then
          IF (NBUG7.LT.2) GOTO 320
          WRITE (77,*) ' VELOCITY ',IK,JK
          DO 310 N = N1,N2
            WRITE (77,3000) (DMV(M,N),M=M1,M2)
  310     CONTINUE
        ENDIF
C
  320   CONTINUE
  330 CONTINUE
         endif
C 	 
          close(18)
          if(ndip.eq.1)deallocate(DML)
          if(ndip.ne.1)deallocate(DMV)
          NBUG7=0
C
C
C
C
C
       if((ndip.eq.1).and.(iam.eq.0))then
        allocate(bbutl(mmnp2,nchan),stat=ierr)
        if(ierr.ne.0)write(0,*)'failure to allocate bbutl in dmat2'
        goto 335
       endif
c
       if((ndip.ne.1).and.(iam.eq.0))then
       allocate(bbutv(mmnp2,nchan),stat=ierr)
       if(ierr.ne.0)write(0,*)'failure to allocate bbutv in dmat2'
       endif
C
C
C   read in LHS Buttle correction into BBUTL,BBUTV
C
 335   CONTINUE
       call comm_barriersub()
C
      if(iam.eq.0)then
      M2 = 0
      DO 240 MCH = 1,MCHAN
        if(ndip.eq.1)then
        READ (ITAPE) ((BBUTL(M,N),N=1,NCHAN),M=M2+1,M2+NRANG2)
        READ (ITAPE) ((dumm,N=1,NCHAN),M=M2+1,M2+NRANG2)
	else
        READ (ITAPE) ((dumm,N=1,NCHAN),M=M2+1,M2+NRANG2)
        READ (ITAPE) ((BBUTV(M,N),N=1,NCHAN),M=M2+1,M2+NRANG2)
	endif	
        M2 = M2 + NOTERM
  240 CONTINUE
      IF (M2.LT.MMNP2) THEN
        if(ndip.eq.1)then
        READ (ITAPE) ((BBUTL(M,N),N=1,NCHAN),M=M2+1,MMNP2)
        READ (ITAPE) ((dumm,N=1,NCHAN),M=M2+1,MMNP2)
	else
        READ (ITAPE) ((dumm,N=1,NCHAN),M=M2+1,MMNP2)
        READ (ITAPE) ((BBUTV(M,N),N=1,NCHAN),M=M2+1,MMNP2)	
	endif
      ENDIF
C
C   multiply LHS V by Buttle correction, overwriting BBUTL and BBUTV
C
       KKCOUNT=0
C
       KKSPLIT=NRANG2
C
       kkmod=mmnp2/kksplit
C
       allocate(kkstart(0:kksplit-1))
       allocate(kkfinal(0:kksplit-1))
        do JJ=0,KKSPLIT-1
        KKSTART(JJ)=1+kkmod*JJ
        KKFINAL(JJ)=kkmod*(JJ+1)
        enddo
        KKFINAL(KKSPLIT-1)=MMNP2
C
C
       if(mmnp2.lt.100)then
       kksplit=1
       kkstart(0)=1
       kkfinal(0)=mmnp2
       endif 
       
       allocate(DUM3(MMNP2,NCHAN),stat=ierr)
       if(ierr.ne.0)write(0,*)'failure to allocate dum3' 
       
       do jjj=1,nchan
        do kkk=1,mmnp2
	 dum3(kkk,jjj)=0.0d0
	enddo
       enddo	      


 85   continue
      iistart=kkstart(kkcount)
      iifinal=kkfinal(kkcount)
             
         allocate(tmpa(mmnp2))
         allocate(VI(MMNP2,iistart:iifinal),stat=ierr)
         if(ierr.ne.0)write(0,*)'failure to allocate vi'
      OPEN (UNIT=8,FILE=sym1,STATUS='OLD',FORM='UNFORMATTED')
         DO 5555 NO = 1,MMNP2 
          READ(8)(TMPA(N),N=1,MMNP2)
          DO 6666 N=iistart,iifinal
          VI(NO,N)=TMPA(N)
 6666   CONTINUE
 5555   CONTINUE
         if(allocated(tmpa))deallocate(TMPA)
C
C
C
C
C      DO 290 N = 1,NCHAN
C        DO 250 MP = 1,MMNP2
C         DUM1(MP) = 0.0D0
C  250   CONTINUE
C        DO 270 M = 1,MMNP2
C          DO 260 MP = 1,MMNP2
C          DUM1(MP) = DUM1(MP) + VI(MP,M)*BBUTL(M,N)
C  260     CONTINUE
C  270   CONTINUE
C        DO 280 MP = 1,MMNP2
C        BBUTL(MP,N) = DUM1(MP)
C  280   CONTINUE
C  290 CONTINUE
C
      if(ndip.eq.1)then
      DO 4003 J = 1,NCHAN
        DO 3003 K = iistart,iifinal
          DCON=BBUTL(K,J)
          DO 2003 I = 1,MMNP2
            DUM3(I,J) = VI(I,K)*DCON + DUM3(I,J)
 2003     CONTINUE
 3003   CONTINUE
 4003  CONTINUE
C
      else
      DO 4004 J = 1,NCHAN
        DO 3004 K = iistart,iifinal
           DCON=BBUTV(K,J)
          DO 2004 I = 1,MMNP2
            DUM3(I,J) = VI(I,K)*DCON + DUM3(I,J)
 2004     CONTINUE
 3004   CONTINUE
 4004  CONTINUE
C
      endif
      if(allocated(vi))deallocate(vi,stat=ierr)
      if(ierr.ne.0)stop 'deallocation fails for vi at bbut'
C      write(0,*)iistart,iifinal
C
       kkcount=kkcount+1
       if(kkcount.eq.(kksplit))then
       if(allocated(kkstart))deallocate(kkstart,kkfinal)
       close(8)
       continue
       else
       close(8)
       goto 85
       endif
C
       if(ndip.eq.1)then
       DO 291 N = 1,NCHAN
        DO 292 M = 1,MMNP2
        BBUTL(M,N)=DUM3(M,N)
  292  continue
  291  continue
       else
       DO 10291 N = 1,NCHAN
        DO 10292 M = 1,MMNP2
        BBUTV(M,N)=DUM3(M,N)
10292  continue
10291  continue
       endif
C       
       if(allocated(dum3))deallocate(DUM3,stat=ierr)
       if(ierr.ne.0)write(0,*)'failure to allocate dum3'
C
C
       endif      
  340  CONTINUE
       deallocate(dum1,dum2)
C
        call comm_barriersub()
C
C       if(iam.eq.0)then
C       write(78,*)((ABUTL(I,J),I=1,MNP2),J=1,MCHAN)
C       write(79,*)((BBUTL(I,J),I=1,MMNP2),J=1,NCHAN)
C       endif
C
        if(iam.eq.0)write(88,*)'END OF DMAT2'
C
C
 3000 FORMAT (8F10.6)
      END
      SUBROUTINE READ1
      use comm_interface
      use param1
      use ciinput
      use dbut1
      use dipoleorder
      use part
C
C  READS DATA INDEPENDENT OF SLPI FROM R-MATRIX FILE, FILA
C
C  THE FOLLOWING DATA ARE READ _
C       NZ = NUCLEAR CHARGE
C       NELC = NUMBER OF ELECTRONS IN TARGET
C       NAST = NUMBER OF TARGET STATES
C       LRANG2 = TOTAL NUMBER OF SMALL L/BIG K VALUES
C       LAMAX = MAXIMUM LAMBDA FOR MULTIPOLE POTENTIALS
C       RA = R-MATRIX RADIUS
C       BSTO = LOGARITHMIC DERIVATIVE
C       FOR I = 1,NAST -
C               ENAT(I) = TARGET ENERGIES
C               LAT(I) = TARGET ORBITAL ANGULAR MOMENTA
C               ISAT(I) = VALUES OF (2*S+1) FOR TARGET STATES
C       FOR I = 1,3 AND L = 1,LRANG2 -
C               COEFF(I,L) = BUTTLE CORRECTION FITS
C       FOR I=1,NBUTD(L) FOR L=1,LRANG2 DARC BUTTLE CORRECTION
C               EBUTD(I,L),CBUTD(I,L)
C
      IMPLICIT REAL*8 (A-H,O-Z)
      character*7 :: chara
      character*8 :: charb
      character*9 :: charc             
C
C
      LOGICAL EX
C
C     COMMON/CINPUT/
C     1 LAMAX,LRANG2,LRGL2,MNP2,NAST,NCHAN,NELC,NPTY2,NSPN2,NZ,MORE2,
C     2 ISAT(MZTAR),LAT(MZTAR),L2P(MZCHF),NCONAT(MZTAR),KJ(MZCHF),KFLAG
C      COMMON/CINPTX/BSTO,RA,
C     4 CF(MZCHF,MZCHF,MZLMX),COEFF(3,MZLR2),ENAT(MZTAR)
C     5 ,WMAT(MZMNP,MZCHF),VALUE(MZMNP)
C      COMMON/CNTRL/IPRINT,IPRKM,IWORD,IRAD,IPERT,NPERT,IPRTSW
C      COMMON/DBUT/EBUTD(MZNR2,MZLR2),CBUTD(MZNR2,MZLR2),NBUTD(MZNR2)
C     X           ,K2P(MZCHF)
c      COMMON/PART/EIGENS(MZNR2,MZLR2),ENDS(MZNR2,MZLR2),SI(MZCHF),
c     X            TRACE,NRANG1(MZLR2),NRANG2,IPRCENT
C
C
C
C
C  READ AND DIMENSION CHECKS
C
      IF(IWORD.EQ.2)GO TO 50
      READ(10)NELC,NZ,LRANG2,LAMAX,NAST,RA,BSTO
C
      IF(LRANG2.LT.0)THEN     !FROM DARC DSTG2 VIA DTO3.
        LRANG2=-LRANG2
        KFLAG=-1
      ENDIF
      IF(NAST.LT.0)THEN
        IPRCENT=0
        NAST=-NAST
      ELSE
        IPRCENT=100
      ENDIF
C

C     Check to see whether we have a LS/BP case 
C
C      if(iam.eq.0)write(6,*)NELC,NZ,LRANG2,LAMAX,NAST,RA,BSTO
C     Need NRANG2 
C
               if(nsym.eq.1)then
C
      IF(IPRCENT.eq.100)then
      INQUIRE(FILE='sizeH.dat',EXIST=EX)
      if(.not.EX)then
      if(masteriam.eq.0) write(6,*)'Breit-Pauli or DARC calculation'
      goto 1500
      else
      if(masteriam.eq.0) write(6,*)'LS or MVD calculation'
 100  OPEN (UNIT=37,FILE='sizeH.dat',STATUS='OLD',FORM='FORMATTED')
      read(37,3010,err=100,end=1000)charb,IDUM1,charc,IDUM2,chara,IDUM3
      NRANG2=IDUM2/IDUM1
 1000 close(37)
      endif
3010  FORMAT (a8,I5,a9,I7,a7,I7)
      goto 2000      
      ENDIF
C
C
 1500 IF(IPRCENT.eq.100)then
      INQUIRE(FILE='sizeBP.dat',EXIST=EX)
      if(.not.EX)then
       if(masteriam.eq.0)then
      write(6,*)'                                       '
      write(6,*)'need sizeBP.dat to work out NRANG2 size'
      write(6,*)' as no sizeH.dat file is present either     '
      write(6,*)'                                       ' 
       endif
      stop
      else
 110  OPEN (UNIT=37,FILE='sizeBP.dat',STATUS='OLD',FORM='FORMATTED')
      read(37,995)IDUM1,IDUM2,IDUM3,A5,IDUM4,A4,IDUM5
      NRANG2=IDUM2/IDUM1
      close(37)
      endif
      ENDIF
C
 995  FORMAT(3I7,A5,I7,A4,I7)
 2000 continue
C
               endif
C
C
      IF(NAST.GT.MZTAR)THEN
        WRITE(6,610)NAST
        STOP '*** DIMENSION EXCEEDED: INCREASE MZTAR'
      ENDIF
C
      allocate(ENAT(NAST),LAT(NAST),ISAT(NAST))
      allocate(COEFF(3,MZLR2))
      READ(10)(ENAT(I),I=1,NAST)
      READ(10)(LAT(I),I=1,NAST)
      READ(10)(ISAT(I),I=1,NAST)
C 
      IF(LRANG2.GT.MZLR2)THEN
        WRITE(6,620)LRANG2,MZLR2
        STOP '*** DIMENSION EXCEEDED: INCREASE MZLR2'
      ENDIF
C
      READ(10)((COEFF(I,L),I=1,3),L=1,LRANG2)
      deallocate(COEFF)
C
C      IF(IPRCENT.NE.100)THEN              !FOR PARTITIONED
C        READ(10)IPRCENT,NRANG2
C        IF(NRANG2.GT.MZNR2)THEN
C          WRITE(6,625)NRANG2,MZNR2
C          STOP  '*** DIMENSION EXCEEDED: INCREASE MZNR2'
C        ENDIF
C        DO L = 1,LRANG2
C          READ (10) (EIGENS(N,L),N=1,NRANG2)
C          READ (10) (ENDS(N,L),N=1,NRANG2)
C        ENDDO
C      ENDIF
C
      INQUIRE(FILE='DBUT.DAT',EXIST=EX)
      IF(EX)THEN
        IWORD=-1
        IB=11
        OPEN(11,FILE='DBUT.DAT',FORM='UNFORMATTED')
        GO TO 51
      ENDIF
C
      RETURN
C
C DARC
   50 READ(10)
      READ(10)NELC,NZ,NRANG2,LRANG2,NAST,RA,BSTO
C      write(6,*)'In read1 nrang2=',NRANG2
C
      IF(NAST.GT.MZTAR)THEN
        WRITE(6,610)NAST
        STOP '*** DIMENSION EXCEEDED: INCREASE MZTAR'
      ENDIF
C
      READ(10)(ENAT(I),I=1,NAST)
      READ(10)(LAT(I),I=1,NAST)
C
      DO I=1,NAST
        LAT(I)=ABS(LAT(I))-1
        ISAT(I)=0
      ENDDO
C
      IF(LRANG2.GT.MZLR2)THEN
        WRITE(6,620)LRANG2,MZLR2
        STOP '*** DIMENSION EXCEEDED: INCREASE MZLR2'
      ENDIF
C
      IB=10
C
   51 continue
      allocate(EBUTD(MZNR2,MZLR2),CBUTD(MZNR2,MZLR2))
      allocate(NBUTD(MZNR2),K2P(MZCHF))
      DO L=1,LRANG2
        READ(IB)NBUTD(L)
        IF(NBUTD(L).GT.MZNR2)THEN
          WRITE (6,699) NBUTD(L),MZNR2
          STOP '*** DIMENSION EXCEEDED: INCREASE MZNR2'
        ENDIF
        READ(IB)(EBUTD(I,L),I=1,NBUTD(L))
        READ(IB)(CBUTD(I,L),I=1,NBUTD(L))
      ENDDO
C
      IF(IB.EQ.11)CLOSE(11)
C
      deallocate(ENAT,LAT,ISAT)
      RETURN
C
  610 FORMAT(///20X,'TOO MANY TARGET STATES'//
     1 10X,'VALUE READ FOR NAST IS ',I3//
     2 10X,'MAXIMUM ALLOWED BY DIMENSIONS IS MZTAR'//)
  620 FORMAT(///20X,'TOO MANNY BUTTLE COEFFICIENTS'//
     1 10X,'VALUE READ FOR LRANG2 IS ',I3//
     2 10X,'MAXIMUM VALUE ALLOWED BY DIMENSIONS IS MZLR2 =',I3//)
  625 FORMAT(//' ******* NRANG2 = ',I4,'  LARGER THAN ',
     + 'MZNR2 = ',I4//)
  699 FORMAT(//' ******* NBUT = ',I4,'  LARGER THAN ',
     + 'MZNR2 = ',I4//)
C
      END
C***************************************************************
C
      SUBROUTINE READ2(IOPT1,nmap,imark)
      use comm_interface
      use param1
      use ciinput
      use dipoleorder
      use part
C
C  READS R-MATRIX DATA FOR ONE SLPI CASE, FROM FILA
C
C  THE FOLLOWING DATA ARE READ -
C       LRGL2 = TOTAL ORBITAL ANGULAR MOMENTUM/ 2J
C       NSPN2 = TOTAL (2*S+1)/0
C       NPTY2 = TOTAL PARITY
C       NCHAN = NUMBER OF CHANNELS
C       MNP2 = NUMBER OF R-MATRIX POLES
C       MORE2 = ZERO TO TERMINATE SLPI CASES
C       FOR I = 1,NAST -
C               NCONAT(I) = NUMBER OF CHANNELS FOR TARGET STATE I
C       FOR I = 1,NCHAN -
C               L2P(I), KJ(I) = SMALL L AND BIG jK FOR CHANNEL I
C                             = K-1, 0 FROM DSTG2/DARC
C       FOR I = 1,NCHAN AND N = 1,NCHAN AND M = 1,LAMAX -
C               CF(I,N,M) = COEFFICIENTS IN MULTIPOLE POTENTIALS
C       FOR I = 1,MNP2 -
C               VALUE(I) = R-MATRIX POLE ENERGIES
C       FOR K = 1,NCHAN AND I = 1,MNP2 -
C               WMAT(I,K) = R-MATRIX AMPLITUDES
C
      IMPLICIT REAL*8 (A-H,O-Z)
C
      CHARACTER*1 NUM(0:9)
      CHARACTER*8 IORD
      DATA NUM/'0','1','2','3','4','5','6','7','8','9'/
C
C
      PARAMETER (MXTST=(MZTAR*MZTAR+MZTAR)/2)
C
      COMMON/CHAN/ECH(MZCHF),EPS(MZCHF),FKNU(MZCHF),CC(MZCHF)
     1  ,RINF(MZCHF),ITARG(MZCHF),LLCH(MZCHF),NCHF,NCHOP,NCHOP1
      COMMON/CDEC/ARAD(MXTST),ARDEC(MZTAR),SLIN(MXTST),IRDEC,IEND
     X,IPAR(MZTAR),NEWAR
      COMMON/CDEGEN/ENATR(MZTAR),NASTD,NASTR,NLEV(MZTAR),NCNATR(MZTAR)
     X,IWD(MZTAR),IWT
      COMMON/CNTRL/IPRINT,IPRKM,IWORD,IRAD,IPERT,NPERT,IPRTSW
      COMMON/DBUT/EBUTD(MZNR2,MZLR2),CBUTD(MZNR2,MZLR2),NBUTD(MZNR2)
     X           ,K2P(MZCHF)
      COMMON/NRBLMX/LMX
c      COMMON/PART/EIGENS(MZNR2,MZLR2),ENDS(MZNR2,MZLR2),SI(MZCHF),
c     X            TRACE,NRANG1(MZLR2),NRANG2,IPRCENT
      COMMON /RECORD/ILSPI(4,MZSLP),IORDER(MZCHF,MZSLP)
C
C
C
C
C  READ AND DIMENSION CHECKS
C
      IF(IWORD.NE.2)THEN
        READ(10,END=999)LRGL2,NSPN2,NPTY2,NCHAN,MNP2,MORE2
      ELSE                               !DARC
        READ(10,END=999)LRGL2,NPTY2,NCHAN,MNP2,NCFGP,LAMAX
        MORE2=1
        LRGL2=ABS(LRGL2)-1
        NSPN2=0
        IF(NPTY2.EQ.-1)THEN
          NPTY2=1
        ELSE
          NPTY2=0
        ENDIF
        KFLAG=-2
      ENDIF
C
C
       idipsym(1,nmap)=NSPN2
       idipsym(2,nmap)=LRGL2
       idipsym(3,nmap)=NPTY2
       idipsym(4,nmap)=(100*NSPN2+LRGL2+1)* (1-2*MOD(NPTY2,2))
       idipsym(5,nmap)= abs(MNP2)
       idipsym(6,nmap)= NCHAN
       idipsym(7,nmap)= 0
C
C      WRITE(6,*)LRGL2,NSPN2,NPTY2
 10     allocate(L2P(MZCHF),KJ(MZCHF),stat=ierr1)
      if(ierr1.ne.0)stop 'allocation fails for L2P or KJ -READ2 '
      allocate(NCONAT(NAST),stat=ierr2)
      if(ierr2.ne.0)stop 'allocation fails for NCONAT -READ2 '
C      write(6,*)'NAST =',NAST
      READ(10)(NCONAT(I),I=1,NAST)     ! NAST?

      IF(NCHAN.GE.MZCHF)THEN            !.GE. FOR SR.LU
        WRITE(6,600)NSPN2,LRGL2,NPTY2,NCHAN
        STOP  '*** DIMENSION EXCEEDED: INCREASE MZCHF'
      ENDIF
C
C
C
C
C
C 
      ILSPI(1,nmap) = (100*NSPN2+LRGL2+1)* (1-2*MOD(NPTY2,2))
      ILSPI(2,nmap) =  abs(MNP2)
      ILSPI(3,nmap) =  NCHAN
      ILSPI(4,nmap) =  0
C      ILSPI(4,INC) =  -MEM1
C
       if((iam.eq.0).and.(imark.eq.1))then
        write(88,*) '                                          '
        write(88,*) '                                          '
       IF (mod(nmap,2).eq.1) write(88,*)'symmetry dimension =',
     X                 ILSPI(2,nmap),'   sym number=',nmap,
     X'                                      and nchan=',nchan,
     X' in subworld',icolour   
       IF (mod(nmap,2).eq.0) write(88,*)'symmetry dimension =',
     X                 ILSPI(2,nmap),'   sym number=',nmap,
     X'                                      and nchan=',nchan,
     X' in subworld',icolour
       endif
C
        i1=iorderdip(nmap,2)/10 
        i2=iorderdip(nmap,2)-10*i1
        IORD='IORDER'//NUM(i1)//NUM(i2)
C
C
       IF(mod(iorderdip(nmap,2),2).eq.1)then
       INC=1
       OPEN (UNIT=58,FILE=IORD,STATUS='OLD',FORM='FORMATTED')
       DO I=1,NCHAN
        READ(58,*)IORDER(I,INC)
       ENDDO
       CLOSE(58)
c
       else
c
       OPEN (UNIT=59,FILE=IORD,STATUS='OLD',FORM='FORMATTED')
       INC=2 
       DO I=1,NCHAN
        READ(59,*)IORDER(I,INC)
       ENDDO
       CLOSE(59)
c
       ENDIF

C
C
C
 20     IF(NSPN2.NE.0)READ(10)(L2P(I),I=1,NCHAN)
      IF(NSPN2.EQ.0)THEN
        IF(KFLAG.EQ.1)READ(10,ERR=997)(L2P(I),I=1,NCHAN),
     X                                      (KJ(I),I=1,NCHAN)
        IF(KFLAG.NE.1)READ(10)(L2P(I),I=1,NCHAN)
      ENDIF
C
      IF(LAMAX.GT.MZLMX)THEN
        MLAMRD=MZLMX
        IF(LMX.GT.MLAMRD)WRITE(6,620)LMX
      ELSE
        MLAMRD=LAMAX
      ENDIF
C
C      allocate(CF(MZCHF,MZCHF,MZLMX))
      IF(LAMAX.GT.0)
     XREAD(10)(((DUM,I=1,NCHAN),N=1,NCHAN),M=1,MLAMRD)
      IF(LAMAX.GT.MLAMRD)
     XREAD(10)(((DUM,I=1,NCHAN),N=1,NCHAN),M=1,LAMAX-MLAMRD)
C      deallocate(CF)
C
c      IF(MNP2.GT.MZMNP)THEN
c        WRITE(6,610)NSPN2,LRGL2,NPTY2,MNP2
c        STOP '*** DIMENSION EXCEEDED: INCREASE MZMNP'
c      ENDIF
C
      IF(IPRCENT.NE.100)READ(10)TRACE

      allocate(WMAT(abs(MNP2),NCHAN),VALUE(abs(MNP2)))

  12  continue

      IF(MNP2.GE.0)THEN
        READ(10)(VALUE(I),I=1,MNP2)
        READ(10)((WMAT(I,K),K=1,NCHAN),I=1,MNP2)
c        write(6,*)'standard wmat read'
      ELSE                                   !DIVIDED WMAT - CPB
        MNP2=abs(MNP2)
        READ(10)(VALUE(I),I=1,MNP2)
        KK=0
  13    READ(10,iostat=ierr)ILOW,IUPPER,NDIV
c        write(6,*)'get read2 div',ILOW,IUPPER,NDIV
        READ(10)((WMAT(I-ILOW+1+KK,K),K=1,NCHAN),I=ILOW,IUPPER)
        KK=KK+IUPPER-ILOW+1
        IF(ILOW.NE.1) GO TO 13
      ENDIF

       MNP2=abs(MNP2)

      deallocate(WMAT,VALUE)
C
      IF(IWORD.EQ.2)THEN                     !DARC
        DO I=1,NCHAN
          L2P(I)=2*L2P(I)                    !2*kappa
          IF(L2P(I).LT.0)L2P(I)=-L2P(I)-1    !K, analogue of L
          L2P(I)=L2P(I)-1                    !K-1
        ENDDO
      ENDIF
      IF(KFLAG.LT.0)THEN                     !L2P(I)=K-1
        DO I=1,NCHAN
          LLCH(I)=(L2P(I)+1)/2               !CHANNEL ORB ANG 
          K2P(I)=LLCH(I)
          IF(LLCH(I)*2.NE.L2P(I)+1)K2P(I)=-K2P(I)-1   !KAPPA
          KJ(I)=0                            !AS JJ COUPLING
        ENDDO
      ELSE
        DO I=1,NCHAN
          LLCH(I)=L2P(I)                     !CHANNEL ORB ANG
        ENDDO
      ENDIF
C
C  GROUP TOGETHER CHANNELS BELONGING TO DEGENERATE LEVELS
C  AND PRESERVE THE READ VALUES OF NCONAT IN /CDEGEN/
      IF(IOPT1.GT.9) NASTR=NAST
      IF(NASTD.GT.0)THEN
        DO I=1,NASTR
          NCNATR(I)=NCONAT(I)
        ENDDO
        IF(IOPT1.LT.10)THEN
          N1=1
          DO I=1,NASTD
            NCON=0
            N2=NLEV(I)+N1-1
            DO IN=N1,N2
              NCON=NCON+NCNATR(IN)
            ENDDO
            NCONAT(I)=NCON
            N1=N2+1
          ENDDO
        ENDIF
      ENDIF
C
C  TARGET PARITIES
C
      K=1
      DO I=1,NAST
        IF(NCONAT(I).NE.0)THEN
          IF(IPAR(I).LT.0)IPAR(I)=ABS(NPTY2-LLCH(K)+2*(LLCH(K)/2))
          K=K+NCONAT(I)
        ENDIF
      ENDDO
C
      IF (NSPN2.EQ.0.AND.KFLAG.EQ.0) THEN
C
C      RECOVER THE K CHANNEL NUMBERS (BASED ON SR.NJCHAN) IF
C      NOT READ FROM H.DAT
C
      NJCHA = 0
C
C      LOOP OVER THE TARGET STATES.
C
      DO 4 I = 1,NAST
C
C      LOOP OVER THE ONLY TWO POSSIBLE 2K VALUES, THAT IS LRGL2-1,LRGL2+1
C
        IFIN = 3
        IF (LRGL2.EQ.0) IFIN = 1
        DO 3 K = 1,IFIN,2
          JK = ABS(LRGL2-2+K)
C
C      RECOVER THE RANGE OF L(INCIDENT) VALUES.
C
          LMIN = ABS(LAT(I)-JK)
          LMAX = LAT(I) + JK
C
C      CHECK IF LMIN IS AN INTEGER.
C
          IF (MOD(LMIN,2).NE.0) GOTO 3
C
C      CHECK THE PARITY OF LMIN.
C
          LP = LMIN/2 + IPAR(I)
          IF (MOD(LP,2).NE.NPTY2) LMIN = LMIN + 2
C
          LP = (LMIN+LMAX)/2
          IF (MOD(LP,2).EQ.1) LMAX = LMAX - 2
          IF (LMIN.GT.LMAX) GOTO 3
C
C      STORE THE 2K-VALUES IN KJ.
C
          LMIN = LMIN + 1
          LMAX = LMAX + 1
          DO 2 L = LMIN,LMAX,4
            NJCHA = NJCHA + 1
            KJ(NJCHA) = JK
    2     CONTINUE
    3   CONTINUE
    4 CONTINUE
C
C END RECOVERY OF K QUANTUM NUMBER
C
      ENDIF
      deallocate(L2P,KJ)
      deallocate(NCONAT)
C
      RETURN
C
  999 MORE2=-777
      RETURN
C
  997 WRITE(6,*)'***SR.READ2 ERROR, TRY SETTING KLAG=0'
      STOP '***SR.READ2 ERROR, TRY SETTING KLAG=0'
C
  600 FORMAT(///20X,'TOO MANY CHANNELS FOR (IS, IL, IP) = (',
     1  3I3,')'//10X,'VALUE READ FOR NCHAN IS ',I4//
     2  10X,'MAXIMUM ALLOWED BY DIMENSIONS IS MZCHF'//)
  610 FORMAT(///20X,'TOO MANY R-MATRIX STATES FOR (IS, IL, IP) = (',
     * 3I3,')'//10X,'VALUE READ FOR MNP2 IS ',I5//    
     3 10X,'MAXIMUM ALLOWED BY DIMENSIONS IS MZMNP'//)
  620 FORMAT(///20X,'WARNING TOO MANY MULTIPOLES REQUESTED VIZ.',I3//
     2 20X,'MAXIMUM ALLOWED BY DIMENSIONS IS MZLMX'//)
C
      END
C***************************************************************
      SUBROUTINE RECOV2(SUBNAM,PARNAM,IDIM,NEED)
      use comm_interface, only : iam,nproc
C
C
C
C-----------------------------------------------------------------------
C
C      THIS ROUTINE IS CALLED ONLY FOR ARRAY OVERFLOW
C
C      SUBNAM = SUBROUTINE NAME
C      PARNAM = PARAMETER NAME
C      IDIM   = CURRENT DIMENSION
C      NEED   = REQUIRED DIMENSION, RETURN NEED=IDIM
C      IPLACE = 0 TO STOP PROGRAM,
C               OTHERWISE RETURN IPLACE=NEED TO THE CALLING ROUTINE.
C
C-----------------------------------------------------------------------
      CHARACTER*6 SUBNAM,PARNAM,PREPRO
C
      COMMON /INFORM/IREAD,IWRITE,IPUNCH
      COMMON /RECOV/IPLACE
C-----------------------------------------------------------------------
      PREPRO = PARNAM
      IF (PREPRO(1:3).EQ.' MZ') PREPRO(1:3) = ' MZ'
      WRITE (IWRITE,3000) SUBNAM,PREPRO,IDIM,PREPRO,NEED
C
      IF (IPLACE.EQ.0) THEN
        WRITE (IWRITE,3010)
        STOP
      ENDIF
C
      IF (IPLACE.LT.0) WRITE (IWRITE,3020)
      IPLACE = NEED
      NEED = IDIM
C
 3000 FORMAT (/' * ARRAY OVERFLOW IN ',
     A       A6/' MUST INCREASE DIMENSION GIVEN BY ',A6,' =',I7,
     B       '  TO AT LEAST ',A6,' =',I9)
 3010 FORMAT (/' PROGRAM TERMINATES IN RECOV2'/)
 3020 FORMAT (/' CHECK TO SEE IF OTHER ARRAYS ARE GOING TO BE EXCEEDED')
      END
